@@ -58,6 +58,7 @@ _IMAGE_IMPORT_BUILDER_EXECUTABLE = 'gce_vm_image_import'
 _IMAGE_EXPORT_BUILDER_EXECUTABLE = 'gce_vm_image_export'
 _OVF_IMPORT_BUILDER_EXECUTABLE = 'gce_ovf_import'
 _OS_UPGRADE_BUILDER_EXECUTABLE = 'gce_windows_upgrade'
+_IMAGE_ONESTEP_IMPORT_BUILDER_EXECUTABLE = 'gce_onestep_image_import'
 
 _DEFAULT_BUILDER_VERSION = 'release'
 
@@ -69,45 +70,45 @@ ROLE_IAM_SERVICE_ACCOUNT_USER = 'roles/iam.serviceAccountUser'
 ROLE_IAM_SERVICE_ACCOUNT_TOKEN_CREATOR = 'roles/iam.serviceAccountTokenCreator'
 ROLE_EDITOR = 'roles/editor'
 
-IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT = frozenset({
+IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT = (
     ROLE_COMPUTE_STORAGE_ADMIN,
     ROLE_STORAGE_OBJECT_VIEWER,
-})
+)
 
-EXPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT = frozenset({
+EXPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT = (
     ROLE_COMPUTE_STORAGE_ADMIN,
     ROLE_STORAGE_OBJECT_ADMIN,
-})
+)
 
-OS_UPGRADE_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT = frozenset({
+OS_UPGRADE_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT = (
     ROLE_COMPUTE_STORAGE_ADMIN,
     ROLE_STORAGE_OBJECT_ADMIN,
-})
+)
 
-IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT = frozenset({
+IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT = (
     ROLE_COMPUTE_ADMIN,
     ROLE_IAM_SERVICE_ACCOUNT_TOKEN_CREATOR,
     ROLE_IAM_SERVICE_ACCOUNT_USER,
-})
+)
 
-EXPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT = frozenset({
+EXPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT = (
     ROLE_COMPUTE_ADMIN,
     ROLE_IAM_SERVICE_ACCOUNT_TOKEN_CREATOR,
     ROLE_IAM_SERVICE_ACCOUNT_USER,
-})
+)
 
-OS_UPGRADE_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT = frozenset({
+OS_UPGRADE_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT = (
     ROLE_COMPUTE_ADMIN,
     ROLE_IAM_SERVICE_ACCOUNT_TOKEN_CREATOR,
     ROLE_IAM_SERVICE_ACCOUNT_USER,
-})
+)
 
 # Used for unit testing as api_tools mocks can only assert on exact matches
 bucket_random_suffix_override = None
 
 
-class FilteredLogTailer(cb_logs.LogTailer):
-  """Subclass of LogTailer that allows for filtering."""
+class FilteredLogTailer(cb_logs.GCSLogTailer):
+  """Subclass of GCSLogTailer that allows for filtering."""
 
   def _PrintLogLine(self, text):
     """Override PrintLogLine method to use self.filter."""
@@ -219,11 +220,18 @@ def _CheckIamPermissions(project_id, cloudbuild_service_account_roles,
       prompt_message = (
           'The "{0}" service is not enabled for this project. '
           'It is required for this operation.\n').format(service_name)
-      console_io.PromptContinue(
+      enable_service = console_io.PromptContinue(
           prompt_message,
           'Would you like to enable this service?',
-          throw_if_unattended=True,
-          cancel_on_no=True)
+          throw_if_unattended=True)
+      if enable_service:
+        services_api.EnableService(project.projectId, service_name)
+      else:
+        log.warning(
+            'If import fails, manually enable {0} before retrying. For '
+            'instructions on enabling services, see '
+            'https://cloud.google.com/service-usage/docs/enable-disable.'
+            .format(service_name))
 
   build_account = 'serviceAccount:{0}@cloudbuild.gserviceaccount.com'.format(
       project.projectNumber)
@@ -249,7 +257,7 @@ def _CheckIamPermissions(project_id, cloudbuild_service_account_roles,
 
   _VerifyRolesAndPromptIfMissing(project_id, build_account,
                                  _CurrentRolesForAccount(policy, build_account),
-                                 cloudbuild_service_account_roles)
+                                 frozenset(cloudbuild_service_account_roles))
 
   current_compute_account_roles = _CurrentRolesForAccount(
       policy, compute_account)
@@ -291,8 +299,7 @@ def _VerifyRolesAndPromptIfMissing(project_id, account, applied_roles,
   add_roles = console_io.PromptContinue(
       message=prompt_message,
       prompt_string='Would you like to add the permissions',
-      throw_if_unattended=True,
-      cancel_on_no=False)
+      throw_if_unattended=True)
 
   if not add_roles:
     return
@@ -628,6 +635,41 @@ def GetRegionFromZone(zone):
   return '-'.join(zone.split('-')[:-1]).lower()
 
 
+def RunOnestepImageImport(args,
+                          import_args,
+                          tags,
+                          output_filter,
+                          release_track,
+                          docker_image_tag=_DEFAULT_BUILDER_VERSION):
+  """Run a build over gce_onestep_image_import on Cloud Build.
+
+  Args:
+    args: An argparse namespace. All the arguments that were provided to this
+      command invocation.
+    import_args: A list of key-value pairs to pass to importer.
+    tags: A list of strings for adding tags to the Argo build.
+    output_filter: A list of strings indicating what lines from the log should
+      be output. Only lines that start with one of the strings in output_filter
+      will be displayed.
+    release_track: release track of the command used. One of - "alpha", "beta"
+      or "ga".
+    docker_image_tag: Specified docker image tag.
+
+  Returns:
+    A build object that either streams the output or is displayed as a
+    link to the build.
+
+  Raises:
+    FailedBuildException: If the build is completed and not 'SUCCESS'.
+  """
+
+  builder = _GetBuilder(args, _IMAGE_ONESTEP_IMPORT_BUILDER_EXECUTABLE,
+                        docker_image_tag, release_track, _GetImageImportRegion)
+  return RunImageCloudBuild(args, builder, import_args, tags, output_filter,
+                            IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT,
+                            IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT)
+
+
 def RunImageExport(args,
                    export_args,
                    tags,
@@ -709,8 +751,9 @@ def RunImageCloudBuild(args, builder, builder_args, tags, output_filter,
   project_id = projects_util.ParseProject(
       properties.VALUES.core.project.GetOrFail())
 
-  _CheckIamPermissions(project_id, cloudbuild_service_account_roles,
-                       compute_service_account_roles,
+  _CheckIamPermissions(project_id,
+                       frozenset(cloudbuild_service_account_roles),
+                       frozenset(compute_service_account_roles),
                        args.compute_service_account
                        if 'compute_service_account' in args else '')
 
@@ -856,8 +899,9 @@ def RunOVFImportBuild(args, compute_client, instance_name, source_uri,
   project_id = projects_util.ParseProject(
       properties.VALUES.core.project.GetOrFail())
 
-  _CheckIamPermissions(project_id, IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT,
-                       IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT)
+  _CheckIamPermissions(project_id,
+                       frozenset(IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT),
+                       frozenset(IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT))
 
   ovf_importer_args = []
   AppendArg(ovf_importer_args, 'instance-names', instance_name)
@@ -957,8 +1001,9 @@ def RunMachineImageOVFImportBuild(args, output_filter, release_track):
   project_id = projects_util.ParseProject(
       properties.VALUES.core.project.GetOrFail())
 
-  _CheckIamPermissions(project_id, IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT,
-                       IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT)
+  _CheckIamPermissions(project_id,
+                       frozenset(IMPORT_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT),
+                       frozenset(IMPORT_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT))
 
   machine_type = None
   if args.machine_type or args.custom_cpu or args.custom_memory:
@@ -1059,9 +1104,10 @@ def RunOsUpgradeBuild(args, output_filter, instance_uri, release_track):
   project_id = projects_util.ParseProject(
       properties.VALUES.core.project.GetOrFail())
 
-  _CheckIamPermissions(project_id,
-                       OS_UPGRADE_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT,
-                       OS_UPGRADE_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT)
+  _CheckIamPermissions(
+      project_id,
+      frozenset(OS_UPGRADE_ROLES_FOR_CLOUDBUILD_SERVICE_ACCOUNT),
+      frozenset(OS_UPGRADE_ROLES_FOR_COMPUTE_SERVICE_ACCOUNT))
 
   # Make OS Upgrade time-out before gcloud by shaving off 2% from the timeout
   # time, up to a max of 5m (300s).
@@ -1100,7 +1146,8 @@ def _GetOSUpgradeRegion(args):  # pylint:disable=unused-argument
   Returns:
     str: region. Can be empty.
   """
-  # TODO(b/168663050)
+  if args.zone:
+    return utils.ZoneNameToRegionName(args.zone)
   return ''
 
 
@@ -1176,6 +1223,72 @@ def AddGuestEnvironmentArg(parser, resource='instance'):
   )
 
 
+def AddAWSImageImportSourceArgs(aws_group):
+  """Adds args for image import from AWS."""
+
+  aws_group.add_argument(
+      '--aws-access-key-id',
+      required=True,
+      help="""\
+          Access key ID for a temporary AWS credential.
+          This ID must be generated using the AWS Security Token Service.
+          """
+  )
+  aws_group.add_argument(
+      '--aws-secret-access-key',
+      required=True,
+      help="""\
+          Secret access key for a temporary AWS credential.
+          This key must be generated using the AWS Security Token Service.
+          """
+  )
+  aws_group.add_argument(
+      '--aws-session-token',
+      required=True,
+      help="""\
+          Session token for a temporary AWS credential. This session
+          token must be generated using the AWS Security Token Service.
+          """
+  )
+  aws_group.add_argument(
+      '--aws-region',
+      required=True,
+      help='AWS region of the image that you want to import.'
+  )
+
+  step_to_begin = aws_group.add_mutually_exclusive_group(
+      required=True,
+      help="""\
+          Specify whether to import from an AMI or disk image.
+          """
+  )
+
+  begin_from_export = step_to_begin.add_group(help="""\
+      If importing an AMI,  specify the following two flags:""")
+  begin_from_export.add_argument(
+      '--aws-ami-id',
+      required=True,
+      help='AWS AMI ID of the image to import.'
+  )
+  begin_from_export.add_argument(
+      '--aws-ami-export-location',
+      required=True,
+      help="""\
+          An AWS S3 bucket location where the converted image file can be
+          temporarily exported to before the import to Cloud Storage."""
+  )
+
+  begin_from_file = step_to_begin.add_group(help="""\
+      If importing a disk image,  specify the following:""")
+  begin_from_file.add_argument(
+      '--aws-source-ami-file-path',
+      help="""\
+          S3 resource path of the exported image file that you want
+          to import.
+          """
+  )
+
+
 def AppendNetworkAndSubnetArgs(args, builder_args):
   """Extracts network/subnet out of CLI args and append for importer.
 
@@ -1209,6 +1322,36 @@ def AddNoAddressArg(parser, operation, docs_url=''):
   parser.add_argument('--no-address', action='store_true', help=help_text)
 
 
+def AddComputeServiceAccountArg(parser, operation, roles):
+  """Adds Compute service account arg."""
+  help_text_pattern = """\
+        A temporary virtual machine instance is created in your project during
+        {operation}.  {operation_capitalized} tooling on this temporary instance
+        must be authenticated.
+
+        A Compute Engine service account is an identity attached to an instance.
+        Its access tokens can be accessed through the instance metadata server
+        and can be used to authenticate {operation} tooling on the instance.
+
+        To set this option,  specify the email address corresponding to the
+        required Compute Engine service account. If not provided, the
+        {operation} on the temporary instance uses the project's default Compute
+        Engine service account.
+
+        At minimum, the specified Compute Engine service account needs to have
+        the following roles assigned:
+        """
+  help_text_pattern += '\n'
+  for role in roles:
+    help_text_pattern += '        * ' + role + '\n'
+
+  parser.add_argument(
+      '--compute-service-account',
+      help=help_text_pattern.format(
+          operation=operation, operation_capitalized=operation.capitalize()),
+  )
+
+
 def _AppendNodeAffinityLabelArgs(
     ovf_importer_args, args, compute_client_messages):
   node_affinities = sole_tenancy_util.GetSchedulingNodeAffinityListFromArgs(
