@@ -25,24 +25,37 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import json
+
 from apitools.base.py import exceptions as apitools_exceptions
+from apitools.base.py import http_wrapper as apitools_http_wrapper
 from apitools.base.py import list_pager
 from apitools.base.py import transfer as apitools_transfer
 
 from googlecloudsdk.api_lib.storage import cloud_api
 from googlecloudsdk.api_lib.storage import errors as cloud_errors
+from googlecloudsdk.api_lib.storage import gcs_metadata_util
+# pylint: disable=unused-import
+# Applies pickling patches:
+from googlecloudsdk.api_lib.storage import patch_gcs_messages
+# pylint: enable=unused-import
 from googlecloudsdk.api_lib.util import apis as core_apis
+from googlecloudsdk.calliope import exceptions as calliope_errors
 from googlecloudsdk.command_lib.storage import storage_url
-from googlecloudsdk.command_lib.storage.resources import gcs_resource_reference
+from googlecloudsdk.command_lib.storage import tracker_file_util
 from googlecloudsdk.command_lib.storage.resources import resource_reference
 from googlecloudsdk.core import exceptions as core_exceptions
+from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
+from googlecloudsdk.core import requests
 from googlecloudsdk.core.credentials import transports
+from googlecloudsdk.core.util import files
+from googlecloudsdk.core.util import retry
 
+import oauth2client
 
 DEFAULT_CONTENT_TYPE = 'application/octet-stream'
-# TODO(b/161346648) Retrieve number of retries from Boto config file.
-DEFAULT_NUM_RETRIES = 23
+MAX_READ_SIZE = 8 * 1024  # 8 KiB.
 
 
 def _catch_http_error_raise_gcs_api_error(format_str=None):
@@ -63,66 +76,39 @@ def _catch_http_error_raise_gcs_api_error(format_str=None):
       format_str=format_str)
 
 
-def _bucket_resource_from_metadata(metadata):
-  """Helper method to generate a BucketResource instance from GCS metadata.
-
-  Args:
-    metadata (messages.Bucket): Extract resource properties from this.
-
-  Returns:
-    BucketResource with properties populated by metadata.
-  """
-  url = storage_url.CloudUrl(scheme=storage_url.ProviderPrefix.GCS,
-                             bucket_name=metadata.name)
-  return gcs_resource_reference.GcsBucketResource(
-      url, etag=metadata.etag, metadata=metadata)
-
-
-def _object_resource_from_metadata(metadata):
-  """Helper method to generate a ObjectResource instance from GCS metadata.
-
-  Args:
-    metadata (messages.Object): Extract resource properties from this.
-
-  Returns:
-    ObjectResource with properties populated by metadata.
-  """
-  if metadata.generation is not None:
-    # Generation may be 0 integer, which is valid although falsy.
-    generation = str(metadata.generation)
-  else:
-    generation = None
-  url = storage_url.CloudUrl(
-      scheme=storage_url.ProviderPrefix.GCS,
-      bucket_name=metadata.bucket,
-      object_name=metadata.name,
-      generation=generation)
-  return gcs_resource_reference.GcsObjectResource(
-      url,
-      creation_time=metadata.timeCreated,
-      etag=metadata.etag,
-      metadata=metadata,
-      metageneration=metadata.metageneration,
-      size=metadata.size)
-
-
-# Disable Apitools' default print callbacks.
 def _no_op_callback(unused_response, unused_object):
+  """Disables Apitools' default print callbacks."""
   pass
 
 
+def get_download_serialization_data(object_resource, progress):
+  """Generates download serialization data for Apitools.
+
+  Args:
+    object_resource (resource_reference.ObjectResource): Used to get metadata.
+    progress (int): Represents how much of download is complete.
+
+  Returns:
+    JSON string for use with Apitools.
+  """
+  serialization_dict = {
+      'auto_transfer': 'False',  # Apitools JSON API feature not used.
+      'progress': progress,
+      'total_size': object_resource.size,
+      'url': object_resource.metadata.mediaLink,  # HTTP download link.
+  }
+  return json.dumps(serialization_dict)
+
+
 class GcsRequestConfig(cloud_api.RequestConfig):
   """Arguments object for requests with custom GCS parameters.
 
   Attributes:
-      decryption_wrapper (CryptoKeyWrapper):
-          utils.encryption_helper.CryptoKeyWrapper for decrypting an object.
-      encryption_wrapper (CryptoKeyWrapper):
-          utils.encryption_helper.CryptoKeyWrapper for encrypting an object.
       gzip_encoded (bool): Whether to use gzip transport encoding for the
           upload.
       max_bytes_per_call (int): Integer describing maximum number of bytes
           to write per service call.
+      md5_hash (str): MD5 digest to use for validation.
       precondition_generation_match (int): Perform request only if generation of
           target object matches the given integer. Ignored for bucket requests.
       precondition_metageneration_match (int): Perform request only if
@@ -132,33 +118,104 @@ class GcsRequestConfig(cloud_api.RequestConfig):
   """
 
   def __init__(self,
-               decryption_wrapper=None,
-               encryption_wrapper=None,
                gzip_encoded=False,
                max_bytes_per_call=None,
+               md5_hash=None,
                precondition_generation_match=None,
                precondition_metageneration_match=None,
                predefined_acl_string=None,
                size=None):
     super(GcsRequestConfig, self).__init__(
-        predefined_acl_string=predefined_acl_string)
-    self.decryption_wrapper = decryption_wrapper
-    self.encryption_wrapper = encryption_wrapper
+        md5_hash=md5_hash,
+        predefined_acl_string=predefined_acl_string,
+        size=size)
     self.gzip_encoded = gzip_encoded
     self.max_bytes_per_call = max_bytes_per_call
     self.precondition_generation_match = precondition_generation_match
     self.precondition_metageneration_match = precondition_metageneration_match
-    self.size = size
+
+  def __eq__(self, other):
+    if not isinstance(other, type(self)):
+      return NotImplemented
+    return (super(GcsRequestConfig, self).__eq__(other) and
+            self.gzip_encoded == other.gzip_encoded and
+            self.max_bytes_per_call == other.max_bytes_per_call and
+            self.precondition_generation_match ==
+            other.precondition_generation_match and
+            self.precondition_metageneration_match ==
+            other.precondition_metageneration_match)
+
+
+class _StorageStreamResponseHandler(requests.ResponseHandler):
+  """Handler for writing the streaming response to the download stream."""
+
+  def __init__(self):
+    """Initializes response handler for requests downloads."""
+    super(_StorageStreamResponseHandler, self).__init__(use_stream=True)
+    self._stream = None
+    self._digesters = {}
+    self._processed_bytes = 0,
+    self._progress_callback = None
+
+  def update_destination_info(self, stream,
+                              digesters=None,
+                              processed_bytes=0,
+                              progress_callback=None):
+    """Updates the stream handler with destination information.
+
+    The download_http_client object is stored on the gcs_api object. This allows
+    resusing the same http_client when the gcs_api is cached using
+    threading.local, which improves performance.
+    Since this same object gets used for mutliple downloads, we need to update
+    the stream handler with the current active download's destination.
+
+    Args:
+      stream (stream): Local stream to write downloaded data to.
+      digesters (dict<HashAlgorithm, hashlib object> | None): For updating hash
+        digests of downloaded objects on the fly.
+      processed_bytes (int): For keeping track of how much progress has been
+        made.
+      progress_callback (func<int>): Accepts processed_bytes and submits
+        progress info for aggregation.
+    """
+    self._stream = stream
+    self._digesters = digesters if digesters is not None else {}
+    self._processed_bytes = processed_bytes
+    self._progress_callback = progress_callback
+
+  def handle(self, source_stream):
+    if self._stream is None:
+      raise ValueError('Stream was not found.')
+
+    # Start reading the raw stream.
+    while True:
+      data = source_stream.read(MAX_READ_SIZE)
+      if data:
+        self._stream.write(data)
+        self._processed_bytes += len(data)
+        if self._progress_callback:
+          self._progress_callback(self._processed_bytes)
+      else:
+        break
+      for hash_object in self._digesters.values():
+        hash_object.update(data)
 
 
 class GcsApi(cloud_api.CloudApi):
   """Client for Google Cloud Storage API."""
 
+  # The API limits the number of objects that can be composed in a single call.
+  # https://cloud.google.com/storage/docs/json_api/v1/objects/compose
+  MAX_OBJECTS_PER_COMPOSE_CALL = 32
+
   def __init__(self):
     self.client = core_apis.GetClientInstance('storage', 'v1')
     self.messages = core_apis.GetMessagesModule('storage', 'v1')
+    self._stream_response_handler = _StorageStreamResponseHandler()
+    self._download_http_client = transports.GetApitoolsTransport(
+        response_encoding=None, response_handler=self._stream_response_handler)
 
-  def _GetProjection(self, fields_scope, message_class):
+  def _get_projection(self, fields_scope, message_class):
     """Generate query projection from fields_scope.
 
     Args:
@@ -186,10 +243,11 @@ class GcsApi(cloud_api.CloudApi):
                     bucket_resource,
                     fields_scope=cloud_api.FieldsScope.NO_ACL):
     """See super class."""
-    projection = self._GetProjection(fields_scope,
-                                     self.messages.StorageBucketsInsertRequest)
+    projection = self._get_projection(fields_scope,
+                                      self.messages.StorageBucketsInsertRequest)
     if not bucket_resource.metadata:
-      bucket_resource.metadata = self.messages.Bucket(name=bucket_resource.name)
+      bucket_resource.metadata = gcs_metadata_util.get_apitools_metadata_from_url(
+          bucket_resource.storage_url)
 
     request = self.messages.StorageBucketsInsertRequest(
         bucket=bucket_resource.metadata,
@@ -197,7 +255,8 @@ class GcsApi(cloud_api.CloudApi):
         projection=projection)
 
     created_bucket_metadata = self.client.buckets.Insert(request)
-    return _bucket_resource_from_metadata(created_bucket_metadata)
+    return gcs_metadata_util.get_bucket_resource_from_metadata(
+        created_bucket_metadata)
 
   @_catch_http_error_raise_gcs_api_error()
   def delete_bucket(self, bucket_name, request_config=None):
@@ -214,19 +273,19 @@ class GcsApi(cloud_api.CloudApi):
   @_catch_http_error_raise_gcs_api_error()
   def get_bucket(self, bucket_name, fields_scope=cloud_api.FieldsScope.NO_ACL):
     """See super class."""
-    projection = self._GetProjection(
-        fields_scope, self.messages.StorageBucketsGetRequest)
+    projection = self._get_projection(fields_scope,
+                                      self.messages.StorageBucketsGetRequest)
     request = self.messages.StorageBucketsGetRequest(
         bucket=bucket_name,
         projection=projection)
 
     metadata = self.client.buckets.Get(request)
-    return _bucket_resource_from_metadata(metadata)
+    return gcs_metadata_util.get_bucket_resource_from_metadata(metadata)
 
   def list_buckets(self, fields_scope=cloud_api.FieldsScope.NO_ACL):
     """See super class."""
-    projection = self._GetProjection(
-        fields_scope, self.messages.StorageBucketsListRequest)
+    projection = self._get_projection(fields_scope,
+                                      self.messages.StorageBucketsListRequest)
     request = self.messages.StorageBucketsListRequest(
         project=properties.VALUES.core.project.GetOrFail(),
         projection=projection)
@@ -243,7 +302,7 @@ class GcsApi(cloud_api.CloudApi):
         global_params=global_params)
     try:
       for bucket in bucket_iter:
-        yield _bucket_resource_from_metadata(bucket)
+        yield gcs_metadata_util.get_bucket_resource_from_metadata(bucket)
     except apitools_exceptions.HttpError as error:
       core_exceptions.reraise(cloud_errors.GcsApiError(error))
 
@@ -254,12 +313,13 @@ class GcsApi(cloud_api.CloudApi):
                    all_versions=None,
                    fields_scope=cloud_api.FieldsScope.NO_ACL):
     """See super class."""
-    projection = self._GetProjection(
-        fields_scope, self.messages.StorageObjectsListRequest)
+    projection = self._get_projection(fields_scope,
+                                      self.messages.StorageObjectsListRequest)
     global_params = None
     if fields_scope == cloud_api.FieldsScope.SHORT:
       global_params = self.messages.StandardQueryParameters()
-      global_params.fields = 'prefixes,items/name,items/size,items/generation'
+      global_params.fields = (
+          'prefixes,items/name,items/size,items/generation,nextPageToken')
 
     object_list = None
     while True:
@@ -282,7 +342,8 @@ class GcsApi(cloud_api.CloudApi):
       # TODO(b/160238394) Decrypt metadata fields if necessary.
       for object_metadata in object_list.items:
         object_metadata.bucket = bucket_name
-        yield _object_resource_from_metadata(object_metadata)
+        yield gcs_metadata_util.get_object_resource_from_metadata(
+            object_metadata)
 
       # Yield prefixes.
       for prefix_string in object_list.prefixes:
@@ -298,18 +359,20 @@ class GcsApi(cloud_api.CloudApi):
         break
 
   @_catch_http_error_raise_gcs_api_error()
-  def delete_object(self,
-                    bucket_name,
-                    object_name,
-                    generation=None,
-                    request_config=None):
+  def delete_object(self, object_resource, request_config=None):
     """See super class."""
     if not request_config:
       request_config = GcsRequestConfig()
 
+    # S3 requires a string, but GCS uses an int for generation.
+    if object_resource.storage_url.generation is not None:
+      generation = int(object_resource.storage_url.generation)
+    else:
+      generation = None
+
     request = self.messages.StorageObjectsDeleteRequest(
-        bucket=bucket_name,
-        object=object_name,
+        bucket=object_resource.storage_url.bucket_name,
+        object=object_resource.storage_url.object_name,
         generation=generation,
         ifGenerationMatch=request_config.precondition_generation_match,
         ifMetagenerationMatch=request_config.precondition_metageneration_match)
@@ -329,8 +392,8 @@ class GcsApi(cloud_api.CloudApi):
     if generation:
       generation = int(generation)
 
-    projection = self._GetProjection(
-        fields_scope, self.messages.StorageObjectsGetRequest)
+    projection = self._get_projection(fields_scope,
+                                      self.messages.StorageObjectsGetRequest)
 
     # TODO(b/160238394) Decrypt metadata fields if necessary.
     try:
@@ -346,7 +409,7 @@ class GcsApi(cloud_api.CloudApi):
               storage_url.ProviderPrefix.GCS, bucket_name, object_name,
               generation).url_string)
       )
-    return _object_resource_from_metadata(object_metadata)
+    return gcs_metadata_util.get_object_resource_from_metadata(object_metadata)
 
   @_catch_http_error_raise_gcs_api_error()
   def patch_object_metadata(self,
@@ -370,15 +433,15 @@ class GcsApi(cloud_api.CloudApi):
                                PredefinedAclValueValuesEnum,
                                request_config.predefined_acl_string)
 
-    projection = self._GetProjection(
-        fields_scope, self.messages.StorageObjectsPatchRequest)
+    projection = self._get_projection(fields_scope,
+                                      self.messages.StorageObjectsPatchRequest)
 
     # Assume parameters are only for identifying what needs to be patched, and
     # the resource contains the desired patched metadata values.
     patched_metadata = object_resource.metadata
     if not patched_metadata:
-      object_resource.metadata = self.messages.Object(
-          name=object_resource.name, bucket=object_resource.bucket)
+      object_resource.metadata = gcs_metadata_util.get_apitools_metadata_from_url(
+          object_resource.storage_url)
 
     request = self.messages.StorageObjectsPatchRequest(
         bucket=bucket_name,
@@ -391,7 +454,7 @@ class GcsApi(cloud_api.CloudApi):
         projection=projection)
 
     updated_metadata = self.client.objects.Patch(request)
-    return _object_resource_from_metadata(updated_metadata)
+    return gcs_metadata_util.get_object_resource_from_metadata(updated_metadata)
 
   @_catch_http_error_raise_gcs_api_error()
   def copy_object(self,
@@ -400,27 +463,86 @@ class GcsApi(cloud_api.CloudApi):
                   progress_callback=None,
                   request_config=None):
     """See super class."""
-    # TODO(b/161900052): Implement resumable copies.
     # TODO(b/161898251): Implement encryption and decryption.
-    del progress_callback
-
     if not request_config:
       request_config = GcsRequestConfig()
 
-    result_metadata = self.client.objects.Copy(
-        self.messages.StorageObjectsCopyRequest(
-            sourceBucket=source_resource.storage_url.bucket_name,
-            sourceObject=source_resource.storage_url.object_name,
-            destinationBucket=destination_resource.storage_url.bucket_name,
-            destinationObject=destination_resource.storage_url.object_name,
-            ifSourceGenerationMatch=(
-                request_config.precondition_generation_match)))
-    return _object_resource_from_metadata(result_metadata)
+    destination_metadata = getattr(destination_resource, 'metadata', None)
+    if not destination_metadata:
+      destination_metadata = gcs_metadata_util.get_apitools_metadata_from_url(
+          destination_resource.storage_url)
+      if source_resource.metadata:
+        gcs_metadata_util.copy_select_object_metadata(source_resource.metadata,
+                                                      destination_metadata)
+
+    if request_config.max_bytes_per_call:
+      max_bytes_per_call = request_config.max_bytes_per_call
+    else:
+      max_bytes_per_call = properties.VALUES.storage.chunk_size.GetInt()
+
+    if request_config.predefined_acl_string:
+      predefined_acl = getattr(
+          self.messages.StorageObjectsRewriteRequest
+          .DestinationPredefinedAclValueValuesEnum,
+          request_config.predefined_acl_string)
+    else:
+      predefined_acl = None
+
+    if source_resource.generation is None:
+      source_generation = None
+    else:
+      source_generation = int(source_resource.generation)
+
+    tracker_file_path = tracker_file_util.get_tracker_file_path(
+        destination_resource.storage_url,
+        tracker_file_util.TrackerFileType.REWRITE, source_resource.storage_url)
+    rewrite_parameters_hash = tracker_file_util.hash_gcs_rewrite_parameters_for_tracker_file(
+        source_resource,
+        destination_resource,
+        destination_metadata,
+        request_config=request_config)
+    try:
+      resume_rewrite_token = tracker_file_util.read_rewrite_tracker_file(
+          tracker_file_path, rewrite_parameters_hash)
+      log.debug('Found rewrite token. Resuming copy.')
+    except files.MissingFileError:
+      resume_rewrite_token = None
+      log.debug('No rewrite token found. Starting copy from scratch.')
+
+    while True:
+      request = self.messages.StorageObjectsRewriteRequest(
+          sourceBucket=source_resource.storage_url.bucket_name,
+          sourceObject=source_resource.storage_url.object_name,
+          destinationBucket=destination_resource.storage_url.bucket_name,
+          destinationObject=destination_resource.storage_url.object_name,
+          object=destination_metadata,
+          sourceGeneration=source_generation,
+          ifGenerationMatch=request_config.precondition_generation_match,
+          ifMetagenerationMatch=(
+              request_config.precondition_metageneration_match),
+          destinationPredefinedAcl=predefined_acl,
+          rewriteToken=resume_rewrite_token,
+          maxBytesRewrittenPerCall=max_bytes_per_call)
+      rewrite_response = self.client.objects.Rewrite(request)
+      processed_bytes = rewrite_response.totalBytesRewritten
+      if progress_callback:
+        progress_callback(processed_bytes)
+
+      if rewrite_response.done:
+        break
+      elif not resume_rewrite_token:
+        resume_rewrite_token = rewrite_response.rewriteToken
+        tracker_file_util.write_rewrite_tracker_file(
+            tracker_file_path, rewrite_parameters_hash,
+            rewrite_response.rewriteToken)
+
+    tracker_file_util.delete_tracker_file(tracker_file_path)
+    return gcs_metadata_util.get_object_resource_from_metadata(
+        rewrite_response.resource)
 
   # pylint: disable=unused-argument
   def _download_object(self,
-                       bucket_name,
-                       object_name,
+                       cloud_resource,
                        download_stream,
                        apitools_download,
                        apitools_request,
@@ -433,8 +555,8 @@ class GcsApi(cloud_api.CloudApi):
     """GCS-specific download implementation.
 
     Args:
-      bucket_name (str): Bucket containing the object.
-      object_name (str): Object name.
+      cloud_resource (resource_reference.ObjectResource): Contains
+          metadata and information about object being downloaded.
       download_stream (stream): Stream to send the object data to.
       apitools_download (apitools.transfer.Download): Apitools object for
           managing downloads.
@@ -469,10 +591,11 @@ class GcsApi(cloud_api.CloudApi):
     # TODO(b/161437904): Add decryption handling.
 
     if start_byte or end_byte:
-      apitools_download.GetRange(additional_headers=additional_headers,
-                                 start=start_byte,
-                                 end=end_byte,
-                                 use_chunks=False)
+      apitools_download.GetRange(
+          additional_headers=additional_headers,
+          start=start_byte,
+          end=end_byte,
+          use_chunks=False)
     else:
       apitools_download.StreamMedia(
           additional_headers=additional_headers,
@@ -481,50 +604,112 @@ class GcsApi(cloud_api.CloudApi):
           use_chunks=False)
     return apitools_download.encoding
 
-  # pylint: disable=unused-argument
+  def _download_object_resumable(self,
+                                 cloud_resource,
+                                 download_stream,
+                                 apitools_download,
+                                 apitools_request,
+                                 compressed_encoding=False,
+                                 decryption_wrapper=None,
+                                 generation=None,
+                                 serialization_data=None,
+                                 start_byte=0,
+                                 end_byte=None):
+    """Wraps _download_object to make it retriable."""
+    # Hack because nonlocal keyword causes Python 2 syntax error.
+    progress_state = {'last_byte_processed': start_byte}
+
+    def _should_retry_resumable_download(exc_type, exc_value, exc_traceback,
+                                         state):
+      converted_error, _ = calliope_errors.ConvertKnownError(exc_value)
+      if isinstance(exc_value, oauth2client.client.HttpAccessTokenRefreshError):
+        if exc_value.status < 500 and exc_value.status != 429:
+          # Not server error or too many requests error.
+          return False
+      elif not isinstance(converted_error, core_exceptions.NetworkIssueError):
+        # Not known transient network error.
+        return False
+
+      start_byte = download_stream.tell()
+      if start_byte > progress_state['last_byte_processed']:
+        # We've made progress, so allow a fresh set of retries.
+        progress_state['last_byte_processed'] = start_byte
+        state.retrial = 0
+      log.debug('Retrying download from byte {} after exception: {}.'
+                ' Trace: {}'.format(start_byte, exc_type, exc_traceback))
+
+      apitools_http_wrapper.RebuildHttpConnections(apitools_download.bytes_http)
+      return True
+
+    def _call_download_object():
+      return self._download_object(
+          cloud_resource,
+          download_stream,
+          apitools_download,
+          apitools_request,
+          compressed_encoding=compressed_encoding,
+          decryption_wrapper=decryption_wrapper,
+          generation=generation,
+          serialization_data=serialization_data,
+          start_byte=start_byte,
+          end_byte=end_byte)
+
+    return retry.RetryOnException(
+        _call_download_object,
+        max_retrials=properties.VALUES.storage.max_retries.GetInt(),
+        # Convert seconds to miliseconds.
+        max_wait_ms=properties.VALUES.storage.max_retry_delay.GetInt() * 1000,
+        should_retry_if=_should_retry_resumable_download)()
+
   @_catch_http_error_raise_gcs_api_error()
   def download_object(self,
-                      bucket_name,
-                      object_name,
+                      cloud_resource,
                       download_stream,
                       compressed_encoding=False,
                       decryption_wrapper=None,
                       digesters=None,
-                      download_strategy=cloud_api.DownloadStrategy.ONE_SHOT,
-                      generation=None,
-                      object_size=None,
+                      download_strategy=cloud_api.DownloadStrategy.RESUMABLE,
                       progress_callback=None,
-                      serialization_data=None,
                       start_byte=0,
                       end_byte=None):
     """See super class."""
     # S3 requires a string, but GCS uses an int for generation.
-    if generation:
-      generation = int(generation)
+    generation = (
+        int(cloud_resource.generation) if cloud_resource.generation else None)
 
-    if not serialization_data:
+    if start_byte and download_strategy == cloud_api.DownloadStrategy.RESUMABLE:
+      # Resuming download.
+      serialization_data = get_download_serialization_data(
+          cloud_resource, start_byte)
+      apitools_download = apitools_transfer.Download.FromData(
+          download_stream,
+          serialization_data,
+          num_retries=properties.VALUES.storage.max_retries.GetInt())
+    else:
       # New download.
+      serialization_data = None
       apitools_download = apitools_transfer.Download.FromStream(
           download_stream,
           auto_transfer=False,
-          total_size=object_size,
-          num_retries=DEFAULT_NUM_RETRIES)
-      apitools_download.bytes_http = transports.GetApitoolsTransport(
-          response_encoding=None)
-    else:
-      # TODO(b/161437901): Handle resumed download.
-      pass
+          total_size=cloud_resource.size,
+          num_retries=properties.VALUES.storage.max_retries.GetInt())
+
+    self._stream_response_handler.update_destination_info(
+        stream=download_stream,
+        digesters=digesters,
+        processed_bytes=start_byte,
+        progress_callback=progress_callback)
+    apitools_download.bytes_http = self._download_http_client
 
     # TODO(b/161460749) Handle download retries.
     request = self.messages.StorageObjectsGetRequest(
-        bucket=bucket_name,
-        object=object_name,
+        bucket=cloud_resource.bucket,
+        object=cloud_resource.name,
         generation=generation)
 
     if download_strategy == cloud_api.DownloadStrategy.ONE_SHOT:
       return self._download_object(
-          bucket_name,
-          object_name,
+          cloud_resource,
           download_stream,
           apitools_download,
           request,
@@ -535,18 +720,26 @@ class GcsApi(cloud_api.CloudApi):
           start_byte=start_byte,
           end_byte=end_byte)
     else:
-      # TODO(b/161437901): Handle resumable download.
-      pass
+      return self._download_object_resumable(
+          cloud_resource,
+          download_stream,
+          apitools_download,
+          request,
+          compressed_encoding=compressed_encoding,
+          decryption_wrapper=decryption_wrapper,
+          generation=generation,
+          serialization_data=serialization_data,
+          start_byte=start_byte,
+          end_byte=end_byte)
 
   # pylint: disable=unused-argument
   def _upload_object(self,
-                     upload_stream,
+                     source_stream,
                      object_metadata,
                      request_config,
                      apitools_strategy=apitools_transfer.SIMPLE_UPLOAD,
                      progress_callback=None,
                      serialization_data=None,
-                     total_size=0,
                      tracker_callback=None):
     # pylint: disable=g-doc-args
     """GCS-specific upload implementation. Adds args to Cloud API interface.
@@ -558,8 +751,6 @@ class GcsApi(cloud_api.CloudApi):
           apitools.base.py.transfer.
       serialization_data (str): Implementation-specific JSON string of a dict
           containing serialization information for the download.
-      total_size (int): Total size of the upload in bytes.
-          If streaming, total size is None.
       tracker_callback (function): Callback that keeps track of upload progress.
 
     Returns:
@@ -592,44 +783,122 @@ class GcsApi(cloud_api.CloudApi):
           predefinedAcl=predefined_acl)
 
     if apitools_strategy == apitools_transfer.SIMPLE_UPLOAD:
-      # One-shot upload.
       apitools_upload = apitools_transfer.Upload(
-          upload_stream,
+          source_stream,
           content_type,
-          total_size=request_config.size,
           auto_transfer=True,
-          num_retries=DEFAULT_NUM_RETRIES,
-          gzip_encoded=request_config.gzip_encoded)
-      # Not settable through the Apitools Upload class constructor.
-      apitools_upload.strategy = apitools_strategy
+          chunksize=properties.VALUES.storage.chunk_size.GetInt(),
+          gzip_encoded=request_config.gzip_encoded,
+          num_retries=properties.VALUES.storage.max_retries.GetInt(),
+          progress_callback=progress_callback,
+          total_size=request_config.size)
 
       result_object_metadata = self.client.objects.Insert(
           request, upload=apitools_upload)
-      return _object_resource_from_metadata(result_object_metadata)
+
+      return gcs_metadata_util.get_object_resource_from_metadata(
+          result_object_metadata)
     else:
       # TODO(b/160998556): Implement resumable upload.
       pass
 
   @_catch_http_error_raise_gcs_api_error()
   def upload_object(self,
-                    upload_stream,
-                    upload_resource,
+                    source_stream,
+                    destination_resource,
                     progress_callback=None,
                     request_config=None):
     """See CloudApi class for function doc strings."""
-    # Doing this as a default argument above can lead to unexpected bugs:
-    # https://docs.python-guide.org/writing/gotchas/#mutable-default-arguments
-    if request_config is None:
-      request_config = GcsRequestConfig()
+    if progress_callback:
+      # Apitools uploads pass response objects to callbacks. Parse these
+      # because our callbacks only expect an int "processed_bytes".
+      def wrapped_progress_callback(response, message_string):
+        del message_string  # Unused.
+        # Range field format: "bytes=0-138989".
+        _, range_values_string = response.info['range'].split('=')
+        start_byte, end_byte = [int(x) for x in range_values_string.split('-')]
+        processed_bytes = end_byte - start_byte
+        progress_callback(processed_bytes)
+    else:
+      wrapped_progress_callback = None
 
-    upload_metadata = self.messages.Object(
-        name=upload_resource.storage_url.object_name,
-        bucket=upload_resource.storage_url.bucket_name)
+    if request_config.size is None:
+      # Size is required so that apitools_transfer can pick the
+      # optimal upload strategy.
+      raise cloud_errors.GcsApiError(
+          'Upload failed due to missing size. Destination: {}'.format(
+              destination_resource.storage_url.url_string))
 
-    return self._upload_object(
-        upload_stream,
-        upload_metadata,
+    validated_request_config = cloud_api.get_provider_request_config(
+        request_config, GcsRequestConfig)
+
+    object_metadata = self.messages.Object(
+        name=destination_resource.storage_url.object_name,
+        bucket=destination_resource.storage_url.bucket_name,
+        md5Hash=validated_request_config.md5_hash)
+
+    upload_result = self._upload_object(
+        source_stream,
+        object_metadata,
         apitools_strategy=apitools_transfer.SIMPLE_UPLOAD,
-        progress_callback=progress_callback,
-        request_config=request_config,
+        progress_callback=wrapped_progress_callback,
+        request_config=validated_request_config,
         serialization_data=None)
+
+    if progress_callback:
+      # Apitools does not always run the callback at the end of an upload.
+      progress_callback(request_config.size)
+
+    return upload_result
+
+  @_catch_http_error_raise_gcs_api_error()
+  def compose_objects(self,
+                      source_resources,
+                      destination_resource,
+                      request_config=None):
+    """See CloudApi class for function doc strings."""
+
+    if not source_resources:
+      raise cloud_errors.GcsApiError(
+          'Compose requires at least one component object.')
+
+    if len(source_resources) > self.MAX_OBJECTS_PER_COMPOSE_CALL:
+      raise cloud_errors.GcsApiError(
+          'Compose was called with {} objects. The limit is {}.'.format(
+              len(source_resources), self.MAX_OBJECTS_PER_COMPOSE_CALL))
+
+    validated_request_config = cloud_api.get_provider_request_config(
+        request_config, GcsRequestConfig)
+
+    source_messages = []
+    for source in source_resources:
+      source_message = self.messages.ComposeRequest.SourceObjectsValueListEntry(
+          name=source.storage_url.object_name)
+      if source.storage_url.generation is not None:
+        generation = int(source.storage_url.generation)
+        source_message.generation = generation
+      source_messages.append(source_message)
+
+    compose_request_payload = self.messages.ComposeRequest(
+        sourceObjects=source_messages,
+        destination=gcs_metadata_util.get_apitools_metadata_from_url(
+            destination_resource.storage_url))
+
+    compose_request = self.messages.StorageObjectsComposeRequest(
+        composeRequest=compose_request_payload,
+        destinationBucket=destination_resource.storage_url.bucket_name,
+        destinationObject=destination_resource.storage_url.object_name,
+        ifGenerationMatch=(
+            validated_request_config.precondition_generation_match),
+        ifMetagenerationMatch=(
+            validated_request_config.precondition_metageneration_match),
+    )
+
+    if validated_request_config.predefined_acl_string is not None:
+      compose_request.destinationPredefinedAcl = getattr(
+          self.messages.StorageObjectsComposeRequest.
+          DestinationPredefinedAclValueValuesEnum,
+          request_config.predefined_acl_string)
+
+    return gcs_metadata_util.get_object_resource_from_metadata(
+        self.client.objects.Compose(compose_request))
