@@ -803,8 +803,13 @@ class ArgList(ArgType):
   DEFAULT_DELIM_CHAR = ','
   ALT_DELIM_CHAR = '^'
 
-  def __init__(self, element_type=None, min_length=0, max_length=None,
-               choices=None, custom_delim_char=None):
+  def __init__(self,
+               element_type=None,
+               min_length=0,
+               max_length=None,
+               choices=None,
+               custom_delim_char=None,
+               visible_choices=None):
     """Initialize an ArgList.
 
     Args:
@@ -814,6 +819,8 @@ class ArgList(ArgType):
       choices: [element_type], a list of valid possibilities for elements. If
           None, then no constraints are imposed.
       custom_delim_char: char, A customized delimiter character.
+      visible_choices: [element_type], a list of valid possibilities for
+          elements to be shown to the user. If None, defaults to choices.
 
     Returns:
       (str)->[str], A function to parse the list of values in the argument.
@@ -823,8 +830,10 @@ class ArgList(ArgType):
     """
     self.element_type = element_type
     self.choices = choices
+    self.visible_choices = (
+        visible_choices if visible_choices is not None else choices)
 
-    if choices:
+    if self.visible_choices:
       def ChoiceType(raw_value):
         if element_type:
           typed_value = element_type(raw_value)
@@ -832,8 +841,9 @@ class ArgList(ArgType):
           typed_value = raw_value
         if typed_value not in choices:
           raise ArgumentTypeError('{value} must be one of [{choices}]'.format(
-              value=typed_value, choices=', '.join(
-                  [six.text_type(choice) for choice in choices])))
+              value=typed_value,
+              choices=', '.join(
+                  [six.text_type(choice) for choice in self.visible_choices])))
         return typed_value
       self.element_type = ChoiceType
 
@@ -1169,12 +1179,17 @@ class UpdateAction(argparse.Action):
         metavar=metavar)
     self.onduplicatekey_handler = onduplicatekey_handler
 
+  def _EnsureValue(self, namespace, name, value):
+    if getattr(namespace, name, None) is None:
+      setattr(namespace, name, value)
+    return getattr(namespace, name)
+
   # pylint: disable=protected-access
   def __call__(self, parser, namespace, values, option_string=None):
 
     if isinstance(values, dict):
       # Get the existing arg value (if any)
-      items = copy.copy(argparse._ensure_value(
+      items = copy.copy(self._EnsureValue(
           namespace, self.dest, collections.OrderedDict()))
       # Merge the new key/value pair(s) in
       for k, v in six.iteritems(values):
@@ -1183,7 +1198,7 @@ class UpdateAction(argparse.Action):
         items[k] = v
     else:
       # Get the existing arg value (if any)
-      items = copy.copy(argparse._ensure_value(namespace, self.dest, []))
+      items = copy.copy(self._EnsureValue(namespace, self.dest, []))
       # Merge the new key/value pair(s) in
       for k in values:
         if k in items:
@@ -1380,6 +1395,44 @@ class StoreOnceAction(argparse.Action):
     setattr(namespace, self.dest, values)
 
 
+def StoreOnceWarningAction(flag_name):
+  """Emits a warning message when a flag is specified more than once.
+
+  The created action is similar to StoreOnceAction. The difference is that
+  this action prints a warning message instead of raising an exception when the
+  flag is specified more than once. Because it is a breaking change to switch an
+  existing flag to StoreOnceAction, StoreOnceWarningAction can be used in the
+  deprecation period.
+
+  Args:
+    flag_name: The name of the flag to apply this action on.
+
+  Returns:
+    An Action class.
+  """
+
+  class Action(argparse.Action):
+    """Emits a warning message when a flag is specified more than once."""
+
+    def OnSecondArgumentPrintWarning(self):
+      log.warning(
+          '"{0}" argument is specified multiple times which will be disallowed '
+          'in future versions. Please only specify it once.'.format(flag_name))
+
+    def __init__(self, *args, **kwargs):
+      self.dest_is_populated = False
+      super(Action, self).__init__(*args, **kwargs)
+
+    def __call__(self, parser, namespace, values, option_string=None):
+      # Make sure no existing arg value exist
+      if self.dest_is_populated:
+        self.OnSecondArgumentPrintWarning()
+      self.dest_is_populated = True
+      setattr(namespace, self.dest, values)
+
+  return Action
+
+
 class _HandleNoArgAction(argparse.Action):
   """This class should not be used directly, use HandleNoArgAction instead."""
 
@@ -1482,11 +1535,47 @@ class YAMLFileContents(object):
     self.validator = validator
 
   def _AssertJsonLike(self, yaml_data):
-    if not yaml.dict_like(yaml_data) or yaml.list_like(yaml_data):
+    if not (yaml.dict_like(yaml_data) or yaml.list_like(yaml_data)):
       raise ArgumentTypeError('Invalid YAML/JSON Data [{}]'.format(yaml_data))
 
+  def _LoadSingleYamlDocument(self, name):
+    """Returns the yaml data for a file or from stdin for a single document.
+
+    YAML allows multiple documents in a single file by using `---` as a
+    separator between documents. See https://yaml.org/spec/1.1/#id857577.
+    However, some YAML-generating tools generate a single document followed by
+    this separator before ending the file.
+
+    This method supports the case of a single document in a file that contains
+    superfluous document separators, but still throws if multiple documents are
+    actually found.
+
+    Args:
+      name: str, The file path to the file or "-" to read from stdin.
+
+    Returns:
+      The contents of the file parsed as a YAML data object.
+    """
+    if name == '-':
+      stdin = console_io.ReadStdin()  # Save to potentially reuse below
+      yaml_data = yaml.load_all(stdin)
+    else:
+      yaml_data = yaml.load_all_path(name)
+    yaml_data = [d for d in yaml_data if d is not None]  # Remove empty docs
+
+    # Return the single document if only 1 is found.
+    if len(yaml_data) == 1:
+      return yaml_data[0]
+
+    # Multiple (or 0) documents found. Try to parse again with single-document
+    # loader so its error is propagated rather than creating our own.
+    if name == '-':
+      return yaml.load(stdin)
+    else:
+      return yaml.load_path(name)
+
   def __call__(self, name):
-    """Load YAML data from file path (name).
+    """Load YAML data from file path (name) or stdin.
 
     If name is "-", stdin is read until EOF. Otherwise, the named file is read.
     If self.validator is set, call it on the yaml data once it is loaded.
@@ -1503,10 +1592,7 @@ class YAMLFileContents(object):
       ValueError: If file content fails validation.
     """
     try:
-      if name == '-':
-        yaml_data = yaml.load(console_io.ReadStdin())
-      else:
-        yaml_data = yaml.load_path(name)
+      yaml_data = self._LoadSingleYamlDocument(name)
       self._AssertJsonLike(yaml_data)
       if self.validator:
         if not self.validator(yaml_data):
