@@ -221,19 +221,23 @@ class Attribute(_Attribute):
   def __eq__(self, other):
     """Overrides."""
     return (super(Attribute, self).__eq__(other)
+            and self.completer == other.completer
             and self.completion_request_params
             == other.completion_request_params
             and self.completion_id_field == other.completion_id_field)
 
   def __hash__(self):
     return super(Attribute, self).__hash__() + sum(
-        map(hash, [six.text_type(self.completion_request_params),
+        map(hash, [six.text_type(self.completer),
+                   six.text_type(self.completion_request_params),
                    self.completion_id_field]))
 
 
 class ResourceSpec(ConceptSpec):
   """Defines a Cloud resource as a set of attributes for argument creation.
   """
+  # TODO(b/67707644): Enable completers by default when confident enough.
+  disable_auto_complete = True
 
   # TODO(b/78851830): update the documentation to use this method.
   @classmethod
@@ -259,14 +263,14 @@ class ResourceSpec(ConceptSpec):
         resource_collection=collection.full_name,
         resource_name=yaml_data['name'],
         api_version=collection.api_version,
-        disable_auto_completers=yaml_data['disable_auto_completers'],
+        disable_auto_completers=yaml_data.get(
+            'disable_auto_completers', ResourceSpec.disable_auto_complete),
         plural_name=yaml_data.get('plural_name'),
         **{attribute.parameter_name: attribute for attribute in attributes})
 
-  # TODO(b/67707644): Enable completers by default when confident enough.
   def __init__(self, resource_collection, resource_name='resource',
-               api_version=None, disable_auto_completers=True, plural_name=None,
-               **kwargs):
+               api_version=None, disable_auto_completers=disable_auto_complete,
+               plural_name=None, **kwargs):
     """Initializes a ResourceSpec.
 
     To use a ResourceSpec, give a collection path such as
@@ -311,6 +315,7 @@ class ResourceSpec(ConceptSpec):
     self._attributes = []
     self._param_names_map = {}
 
+    orig_kwargs = list(six.iterkeys(kwargs))
     # Add attributes.
     anchor = False
     for i, param_name in enumerate(collection_params):
@@ -320,6 +325,7 @@ class ResourceSpec(ConceptSpec):
                                     ResourceParameterAttributeConfig())
       attribute_name = self._AttributeName(param_name, attribute_config,
                                            anchor=anchor)
+
       new_attribute = Attribute(
           name=attribute_name,
           help_text=attribute_config.help_text,
@@ -340,9 +346,11 @@ class ResourceSpec(ConceptSpec):
                                            self._name))
     if kwargs:
       raise ResourceConfigurationError('Resource [{}] was given an attribute '
-                                       'config for unknown attribute(s): [{}]'
+                                       'config for unknown attribute(s): '
+                                       'Expected [{}], Found [{}]'
                                        .format(self._name,
-                                               ', '.join(six.iterkeys(kwargs))))
+                                               ', '.join(collection_params),
+                                               ', '.join(orig_kwargs)))
 
   @property
   def attributes(self):
@@ -451,6 +459,8 @@ class ResourceSpec(ConceptSpec):
       raise InitializationError(
           'The [{}] resource is not properly specified.\n'
           '{}'.format(self.name, six.text_type(e)))
+    except resources.UserError as e:
+      raise InitializationError(six.text_type(e))
 
   def Parse(self, attribute_to_args_map, base_fallthroughs_map,
             parsed_args=None, plural=False, allow_empty=False):
@@ -646,6 +656,7 @@ class ResourceParameterAttributeConfig(object):
     attribute_name = data['attribute_name']
     parameter_name = data['parameter_name']
     help_text = data['help']
+    completer = util.Hook.FromData(data, 'completer')
     completion_id_field = data.get('completion_id_field', None)
     completion_request_params_list = data.get('completion_request_params', [])
     completion_request_params = {
@@ -673,6 +684,7 @@ class ResourceParameterAttributeConfig(object):
         name=attribute_name,
         help_text=help_text,
         fallthroughs=fallthroughs,
+        completer=completer,
         completion_id_field=completion_id_field,
         completion_request_params=completion_request_params,
         parameter_name=parameter_name)
@@ -708,8 +720,9 @@ class ResourceParameterAttributeConfig(object):
     self.attribute_name = name
     self.help_text = help_text
     self.fallthroughs = fallthroughs or []
-    # The completer is always None because neither the surface nor the yaml
-    # schema allow for specifying completers currently.
+    if completer and (completion_request_params or completion_id_field):
+      raise ValueError('Custom completer and auto-completer should not be '
+                       'specified at the same time')
     self.completer = completer
     self.completion_request_params = completion_request_params
     self.completion_id_field = completion_id_field
