@@ -25,7 +25,9 @@ import re
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.container import kubeconfig
+from googlecloudsdk.api_lib.run import container_resource
 from googlecloudsdk.api_lib.run import global_methods
+from googlecloudsdk.api_lib.run import k8s_object
 from googlecloudsdk.api_lib.run import revision
 from googlecloudsdk.api_lib.run import service
 from googlecloudsdk.api_lib.run import traffic
@@ -34,7 +36,7 @@ from googlecloudsdk.api_lib.services import exceptions as services_exceptions
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
-from googlecloudsdk.command_lib.functions.v1.deploy import env_vars_util
+from googlecloudsdk.command_lib.functions.v2.deploy import env_vars_util
 from googlecloudsdk.command_lib.run import config_changes
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
 from googlecloudsdk.command_lib.run import platforms
@@ -69,11 +71,24 @@ _INGRESS_MODES = {
         'Balancing are allowed.'
 }
 
+_SANDBOX_CHOICES = {
+    'gvisor': 'Run the application in a gVisor sandbox.',
+    'minivm': 'Run the application in a mini VM sandbox.',
+}
+
 _DEFAULT_KUBECONFIG_PATH = '~/.kube/config'
 
 _FIFTEEN_MINUTES = 15 * 60
 
 
+def _StripKeys(d):
+  return {k.strip(): v for k, v in d.items()}
+
+
+def _MapLStrip(seq):
+  return [elem.lstrip() for elem in seq]
+
+
 class KubeconfigError(exceptions.Error):
   pass
 
@@ -219,9 +234,20 @@ def AddAllowUnauthenticatedFlag(parser):
       'This may take a few moments to take effect.')
 
 
-def AddAsyncFlag(parser):
+def AddAsyncFlag(parser, default_async_for_cluster=False):
   """Add an async flag."""
-  base.ASYNC_FLAG.AddToParser(parser)
+  if default_async_for_cluster:
+    modified_async_flag = base.Argument(
+        '--async',
+        action=arg_parsers.StoreTrueFalseAction,
+        dest='async_',
+        help="""\
+    Return immediately, without waiting for the operation in progress to
+    complete. Defaults to --no-async for Cloud Run (fully managed) and --async
+    for Cloud Run for Anthos.""")
+    modified_async_flag.AddToParser(parser)
+  else:
+    base.ASYNC_FLAG.AddToParser(parser)
 
 
 def AddEndpointVisibilityEnum(parser, deprecated=False):
@@ -415,6 +441,17 @@ def AddUpdateTrafficFlags(parser, release_track):
       '\'--to-revisions=LATEST=100\'.')
 
 
+def AddSetCloudSQLFlag(parser):
+  """Add only the --set-cloudsql-instances flag."""
+  parser.add_argument(
+      '--set-cloudsql-instances',
+      type=arg_parsers.ArgList(),
+      metavar='CLOUDSQL-INSTANCES',
+      help="""You can specify a name of a Cloud SQL instance if it's in the same
+      project and region as your Cloud Run resource; otherwise specify
+      <project>:<region>:<instance> for the instance.""")
+
+
 def AddCloudSQLFlags(parser):
   """Add flags for setting CloudSQL stuff."""
   repeated.AddPrimitiveArgs(
@@ -483,6 +520,18 @@ def AddMapFlagsNoFile(parser,
       value_metavar=value_metavar)
 
 
+def AddSetEnvVarsFlag(parser):
+  """Add only the --set-env-vars flag."""
+  parser.add_argument(
+      '--set-env-vars',
+      metavar='KEY=VALUE',
+      action=arg_parsers.UpdateAction,
+      type=arg_parsers.ArgDict(
+          key_type=env_vars_util.EnvVarKeyType,
+          value_type=env_vars_util.EnvVarValueType),
+      help='List of key-value pairs to set as environment variables.')
+
+
 def AddMutexEnvVarsFlags(parser):
   """Add flags for creating updating and deleting env vars."""
   # TODO(b/119837621): Use env_vars_util.AddUpdateEnvVarsFlags when
@@ -499,15 +548,15 @@ def AddMemoryFlag(parser):
   parser.add_argument('--memory', help='Set a memory limit. Ex: 1Gi, 512Mi.')
 
 
-def AddCpuFlag(parser):
-  parser.add_argument(
-      '--cpu',
-      help='Set a CPU limit in Kubernetes cpu units.\n\n'
-      'Cloud Run (fully managed) supports values 1, 2 and 4.'
-      '  For Cloud Run (fully managed), 4 cpus also requires a minimum 2Gi'
-      '  `--memory` value.  Examples 2, 2.0, 2000m\n\n'
-      'Cloud Run for Anthos and Knative-compatible Kubernetes clusters support'
-      '  fractional values.  Examples .5, 500m, 2')
+def AddCpuFlag(parser, managed_only=False):
+  help_msg = ('Set a CPU limit in Kubernetes cpu units.\n\n'
+              'Cloud Run (fully managed) supports values 1, 2 and 4.'
+              '  For Cloud Run (fully managed), 4 cpus also requires a minimum '
+              '2Gi `--memory` value.  Examples 2, 2.0, 2000m')
+  if not managed_only:
+    help_msg += ('\n\nCloud Run for Anthos and Knative-compatible Kubernetes '
+                 'clusters support fractional values.  Examples .5, 500m, 2')
+  parser.add_argument('--cpu', help=help_msg)
 
 
 def _ConcurrencyValue(value):
@@ -593,13 +642,24 @@ def AddRevisionSuffixArg(parser):
       'would lead to a revision named \'helloworld-v1\'.')
 
 
+def AddSandboxArg(parser):
+  parser.add_argument(
+      '--sandbox',
+      choices=_SANDBOX_CHOICES,
+      help='Selects the sandbox where the application will run.')
+
+
 def AddVpcConnectorArg(parser):
   parser.add_argument(
-      '--vpc-connector', help='Set a VPC connector for this Service.')
+      '--vpc-connector', help='Set a VPC connector for this resource.')
+
+
+def AddVpcConnectorArgs(parser):
+  AddVpcConnectorArg(parser)
   parser.add_argument(
       '--clear-vpc-connector',
       action='store_true',
-      help='Remove the VPC connector for this Service.')
+      help='Remove the VPC connector for this resource.')
 
 
 def AddEgressSettingsFlag(parser):
@@ -607,13 +667,13 @@ def AddEgressSettingsFlag(parser):
   parser.add_argument(
       '--vpc-egress',
       help='The outbound traffic to send through the VPC connector'
-      ' for this Service. This Service must have a VPC connector to set'
+      ' for this resource. This resource must have a VPC connector to set'
       ' VPC egress.',
       choices={
-          revision.EGRESS_SETTINGS_PRIVATE_RANGES_ONLY:
+          container_resource.EGRESS_SETTINGS_PRIVATE_RANGES_ONLY:
               'Default option. Sends outbound traffic to private IP addresses '
               'defined by RFC1918 through the VPC connector.',
-          revision.EGRESS_SETTINGS_ALL:
+          container_resource.EGRESS_SETTINGS_ALL:
               'Sends all outbound traffic through the VPC connector.'
       })
 
@@ -660,6 +720,13 @@ def AddConfigMapsFlags(parser):
       flag_name='config-maps')
 
 
+def AddLabelsFlag(parser, extra_message=''):
+  """Add only the --labels flag."""
+  labels_util.GetCreateLabelsFlag(
+      extra_message=extra_message, validate_keys=False,
+      validate_values=False).AddToParser(parser)
+
+
 def AddLabelsFlags(parser):
   """Adds update command labels flags to an argparse parser.
 
@@ -668,10 +735,7 @@ def AddLabelsFlags(parser):
   """
   group = parser.add_group()
   add_group = group.add_mutually_exclusive_group()
-  labels_util.GetCreateLabelsFlag(
-      'An alias to --update-labels.',
-      validate_keys=False,
-      validate_values=False).AddToParser(add_group)
+  AddLabelsFlag(add_group, 'An alias to --update-labels.')
   labels_util.GetUpdateLabelsFlag(
       '', validate_keys=False, validate_values=False).AddToParser(add_group)
   remove_group = group.add_mutually_exclusive_group()
@@ -783,6 +847,79 @@ def AddHttp2Flag(parser):
       help='Whether to use HTTP/2 for connections to the service.')
 
 
+def AddParallelismFlag(parser):
+  """Add job parallelism/concurrency flag."""
+  parser.add_argument(
+      '--parallelism',
+      type=arg_parsers.BoundedInt(lower_bound=1),
+      default=1,
+      help='Number of instances that may run concurrently. '
+      'Must be less than or equal to the number of completions.')
+
+
+def AddCompletionsFlag(parser):
+  """Add job number of completions flag."""
+  parser.add_argument(
+      '--completions',
+      type=arg_parsers.BoundedInt(lower_bound=1),
+      default=1,
+      help='Number of instances that must run to completion for the job to be '
+      'considered done. Use this flag to trigger multiple runs of the job.')
+
+
+def AddMaxAttemptsFlag(parser):
+  """Add job max attempts flag to specify number of instance restarts."""
+  parser.add_argument(
+      '--max-attempts',
+      type=arg_parsers.BoundedInt(lower_bound=1),
+      default=6,
+      help='Number of times an instance will be allowed to run in case of '
+      'failure before being failed permanently. This applies per-instance, not '
+      'per-job. If set to 1, instances will only run once and never be '
+      'restarted on failure. If set to any other positive integer N, instances '
+      'will be allowed to restart N-1 times.')
+
+
+def AddWaitForCompletionFlag(parser):
+  """Add job flag to poll until completion on create."""
+  parser.add_argument(
+      '--wait-for-completion',
+      default=False,
+      action='store_true',
+      help='Wait until the job has completed running before finishing polling. '
+      'If not set, polling completes when the job has started.')
+
+
+def AddBinAuthzPolicyFlags(parser, with_clear=True):
+  """Add flags for BinAuthz."""
+  policy_group = parser
+  if with_clear:
+    policy_group = parser.add_mutually_exclusive_group()
+    policy_group.add_argument(
+        '--clear-binary-authorization',
+        default=False,
+        action='store_true',
+        help='Remove any previously set Binary Authorization policy.')
+  policy_group.add_argument(
+      '--binary-authorization',
+      metavar='POLICY',
+      # Don't actually validate the value here, let that happen server-side
+      # so the future change to support named policies will be backwards
+      # compatible with older gcloud versions.
+      help='Binary Authorization policy to check against. This must be set to '
+      '"default".')
+
+
+def AddBinAuthzBreakglassFlag(parser):
+  parser.add_argument(
+      '--breakglass',
+      metavar='JUSTIFICATION',
+      help='Justification to bypass Binary Authorization policy constraints '
+      'and allow the operation. See '
+      'https://cloud.google.com/binary-authorization/docs/using-breakglass '
+      'for more information.')
+
+
 def _HasChanges(args, flags):
   """True iff any of the passed flags are set."""
   return any(FlagIsExplicitlySet(args, flag) for flag in flags)
@@ -851,20 +988,11 @@ def _HasTrafficChanges(args):
 
 def _GetEnvChanges(args):
   """Return config_changes.EnvVarLiteralChanges for given args."""
-  kwargs = {}
-
-  update = args.update_env_vars or args.set_env_vars
-  if update:
-    kwargs['env_vars_to_update'] = update
-
-  remove = args.remove_env_vars
-  if remove:
-    kwargs['env_vars_to_remove'] = remove
-
-  if args.set_env_vars or args.clear_env_vars:
-    kwargs['clear_others'] = True
-
-  return config_changes.EnvVarLiteralChanges(**kwargs)
+  return config_changes.EnvVarLiteralChanges(
+      updates=_StripKeys(
+          getattr(args, 'update_env_vars', None) or args.set_env_vars or {}),
+      removes=_MapLStrip(getattr(args, 'remove_env_vars', None) or []),
+      clear_others=bool(args.set_env_vars or args.clear_env_vars))
 
 
 def _GetScalingChanges(args):
@@ -903,32 +1031,27 @@ def _GetSecretsChanges(args):
   volume_kwargs = {}
   env_kwargs = {}
 
-  update = args.update_secrets or args.set_secrets
-  if update:
-    volume_update = {k: v for k, v in update.items() if _IsVolumeMountKey(k)}
-    if volume_update:
-      volume_kwargs['mounts_to_update'] = volume_update
-    env_update = {k: v for k, v in update.items() if not _IsVolumeMountKey(k)}
-    if env_update:
-      env_kwargs['env_vars_to_update'] = env_update
+  updates = _StripKeys(
+      getattr(args, 'update_secrets', None) or args.set_secrets or {})
+  volume_kwargs['updates'] = {
+      k: v for k, v in updates.items() if _IsVolumeMountKey(k)
+  }
+  env_kwargs['updates'] = {
+      k: v for k, v in updates.items() if not _IsVolumeMountKey(k)
+  }
 
-  remove = args.remove_secrets
-  if remove:
-    volume_remove = [k for k in remove if _IsVolumeMountKey(k)]
-    if volume_remove:
-      volume_kwargs['mounts_to_remove'] = volume_remove
-    env_remove = [k for k in remove if not _IsVolumeMountKey(k)]
-    if env_remove:
-      env_kwargs['env_vars_to_remove'] = env_remove
+  removes = _MapLStrip(getattr(args, 'remove_secrets', None) or [])
+  volume_kwargs['removes'] = [k for k in removes if _IsVolumeMountKey(k)]
+  env_kwargs['removes'] = [k for k in removes if not _IsVolumeMountKey(k)]
 
-  if args.set_secrets or args.clear_secrets:
-    env_kwargs['clear_others'] = True
-    volume_kwargs['clear_others'] = True
+  clear_others = bool(args.set_secrets or args.clear_secrets)
+  env_kwargs['clear_others'] = clear_others
+  volume_kwargs['clear_others'] = clear_others
 
   secret_changes = []
-  if env_kwargs:
+  if any(env_kwargs.values()):
     secret_changes.append(config_changes.SecretEnvVarChanges(**env_kwargs))
-  if volume_kwargs:
+  if any(volume_kwargs.values()):
     secret_changes.append(config_changes.SecretVolumeChanges(**volume_kwargs))
   return secret_changes
 
@@ -938,35 +1061,31 @@ def _GetConfigMapsChanges(args):
   volume_kwargs = {}
   env_kwargs = {}
 
-  update = args.update_config_maps or args.set_config_maps
-  if update:
-    volume_update = {k: v for k, v in update.items() if _IsVolumeMountKey(k)}
-    if volume_update:
-      volume_kwargs['mounts_to_update'] = volume_update
-    env_update = {k: v for k, v in update.items() if not _IsVolumeMountKey(k)}
-    if env_update:
-      env_kwargs['env_vars_to_update'] = env_update
+  updates = _StripKeys(
+      getattr(args, 'update_config_maps', None) or args.set_config_maps or {})
+  volume_kwargs['updates'] = {
+      k: v for k, v in updates.items() if _IsVolumeMountKey(k)
+  }
+  env_kwargs['updates'] = {
+      k: v for k, v in updates.items() if not _IsVolumeMountKey(k)
+  }
 
-  remove = args.remove_config_maps
-  if remove:
-    volume_remove = [k for k in remove if _IsVolumeMountKey(k)]
-    if volume_remove:
-      volume_kwargs['mounts_to_remove'] = volume_remove
-    env_remove = [k for k in remove if not _IsVolumeMountKey(k)]
-    if env_remove:
-      env_kwargs['env_vars_to_remove'] = env_remove
+  removes = _MapLStrip(getattr(args, 'remove_config_maps', None) or [])
+  volume_kwargs['removes'] = [k for k in removes if _IsVolumeMountKey(k)]
+  env_kwargs['removes'] = [k for k in removes if not _IsVolumeMountKey(k)]
 
-  if args.set_config_maps or args.clear_config_maps:
-    env_kwargs['clear_others'] = True
-    volume_kwargs['clear_others'] = True
+  clear_others = bool(args.set_config_maps or args.clear_config_maps)
+  env_kwargs['clear_others'] = clear_others
+  volume_kwargs['clear_others'] = clear_others
 
-  secret_changes = []
-  if env_kwargs:
-    secret_changes.append(config_changes.ConfigMapEnvVarChanges(**env_kwargs))
-  if volume_kwargs:
-    secret_changes.append(
+  config_maps_changes = []
+  if any(env_kwargs.values()):
+    config_maps_changes.append(
+        config_changes.ConfigMapEnvVarChanges(**env_kwargs))
+  if any(volume_kwargs.values()):
+    config_maps_changes.append(
         config_changes.ConfigMapVolumeChanges(**volume_kwargs))
-  return secret_changes
+  return config_maps_changes
 
 
 def PromptToEnableApi(service_name):
@@ -1116,18 +1235,20 @@ def GetConfigurationChanges(args):
         if FlagIsExplicitlySet(args, 'labels') else args.update_labels)
     diff = labels_util.Diff(
         additions=additions,
-        subtractions=args.remove_labels,
-        clear=args.clear_labels)
+        subtractions=args.remove_labels if 'remove_labels' in args else [],
+        clear=args.clear_labels if 'clear_labels' in args else False)
     if diff.MayHaveUpdates():
       changes.append(config_changes.LabelChanges(diff))
   if 'revision_suffix' in args and args.revision_suffix:
     changes.append(config_changes.RevisionNameChanges(args.revision_suffix))
+  if 'sandbox' in args and args.sandbox:
+    changes.append(config_changes.SandboxChange(args.sandbox))
   if 'vpc_connector' in args and args.vpc_connector:
     changes.append(config_changes.VpcConnectorChange(args.vpc_connector))
   if FlagIsExplicitlySet(args, 'vpc_egress'):
     changes.append(
         config_changes.SetTemplateAnnotationChange(
-            revision.EGRESS_SETTINGS_ANNOTATION, args.vpc_egress))
+            container_resource.EGRESS_SETTINGS_ANNOTATION, args.vpc_egress))
   if 'clear_vpc_connector' in args and args.clear_vpc_connector:
     # MUST be after 'vpc_egress' change.
     changes.append(config_changes.ClearVpcConnectorChange())
@@ -1151,22 +1272,41 @@ def GetConfigurationChanges(args):
   if FlagIsExplicitlySet(args, 'tag'):
     # MUST be after 'revision_suffix' change
     changes.append(config_changes.TagOnDeployChange(args.tag))
+  if FlagIsExplicitlySet(args, 'parallelism'):
+    changes.append(config_changes.SpecChange('parallelism', args.parallelism))
+  if FlagIsExplicitlySet(args, 'completions'):
+    changes.append(config_changes.SpecChange('completions', args.completions))
+  if FlagIsExplicitlySet(args, 'max_attempts'):
+    changes.append(config_changes.JobMaxAttemptsChange(args.max_attempts))
+  if FlagIsExplicitlySet(args, 'binary_authorization'):
+    changes.append(
+        config_changes.SetAnnotationChange(
+            k8s_object.BINAUTHZ_POLICY_ANNOTATION,
+            args.binary_authorization))
+  if FlagIsExplicitlySet(args, 'clear_binary_authorization'):
+    changes.append(
+        config_changes.DeleteAnnotationChange(
+            k8s_object.BINAUTHZ_POLICY_ANNOTATION))
+  if FlagIsExplicitlySet(args, 'breakglass'):
+    changes.append(
+        config_changes.SetAnnotationChange(
+            k8s_object.BINAUTHZ_BREAKGLASS_ANNOTATION,
+            args.breakglass))
   return changes
 
 
-def GetService(args):
-  """Get and validate the service resource from the args."""
-  service_ref = args.CONCEPTS.service.Parse()
-  # Valid service names comprise only alphanumeric characters and dashes. Must
+def ValidateResource(resource_ref):
+  """Validate resource name."""
+  # Valid resource names comprise only alphanumeric characters and dashes. Must
   # not begin or end with a dash, and must not contain more than 63 characters.
   # Must be lowercase.
-  service_re = re.compile(r'(?=^[a-z0-9-]{1,63}$)(?!^\-.*)(?!.*\-$)')
-  if service_re.match(service_ref.servicesId):
-    return service_ref
-  raise serverless_exceptions.ArgumentError(
-      'Invalid service name [{}]. Service name must use only lowercase '
-      'alphanumeric characters and dashes. Cannot begin or end with a dash, '
-      'and cannot be longer than 63 characters.'.format(service_ref.servicesId))
+  k8s_resource_name_regex = re.compile(
+      r'(?=^[a-z0-9-]{1,63}$)(?!^\-.*)(?!.*\-$)')
+  if not k8s_resource_name_regex.match(resource_ref.Name()):
+    raise serverless_exceptions.ArgumentError(
+        'Invalid resource name [{}]. The name must use only lowercase '
+        'alphanumeric characters and dashes, cannot begin or end with a dash, '
+        'and cannot be longer than 63 characters.'.format(resource_ref.Name()))
 
 
 def PromptForRegion():
@@ -1270,8 +1410,7 @@ def GetKubeconfig(file_path=None):
     KubeconfigError: if $KUBECONFIG is set but contains no valid paths
   """
   if file_path:
-    return kubeconfig.Kubeconfig.LoadFromFile(
-        files.ExpandHomeDir(file_path))
+    return kubeconfig.Kubeconfig.LoadFromFile(files.ExpandHomeDir(file_path))
   if encoding.GetEncodedValue(os.environ, 'KUBECONFIG'):
     config_paths = encoding.GetEncodedValue(os.environ,
                                             'KUBECONFIG').split(os.pathsep)
@@ -1304,11 +1443,12 @@ def FlagIsExplicitlySet(args, flag):
 def VerifyManagedFlags(args, release_track, product):
   """Raise ConfigurationError if args aren't valid for managed Cloud Run."""
 
-  if product == Product.EVENTS and release_track != base.ReleaseTrack.ALPHA:
+  if product == Product.EVENTS:
     raise serverless_exceptions.ConfigurationError(
-        'The flag --platform={} is only compatible with "gcloud alpha events".'
-        ' Please provide an alternative platform value or use "gcloud alpha '
-        'events".'.format(platforms.PLATFORM_MANAGED))
+        'The flag --platform={0} is not supported. '
+        'Instead of using the flag --platform={0} in "gcloud events", '
+        'run "gcloud eventarc".'
+        .format(platforms.PLATFORM_MANAGED))
 
   error_msg = ('The `{flag}` flag is not supported on the fully managed '
                'version of Cloud Run. Specify `--platform {platform}` or run '
@@ -1347,13 +1487,11 @@ def VerifyManagedFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_GKE]))
 
-  if _HasSecretsChanges(args):
+  if _HasSecretsChanges(args) and release_track != base.ReleaseTrack.ALPHA:
     raise serverless_exceptions.ConfigurationError(
-        error_msg.format(
-            flag='--[update|set|remove|clear]-secrets',
-            platform=platforms.PLATFORM_GKE,
-            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
-                platforms.PLATFORM_GKE]))
+        'The `--[update|set|remove|clear]-secrets` flags are only supported '
+        'in the alpha release track on the fully managed version of Cloud '
+        'Run. Use `gcloud alpha` to enable these flags.')
 
   if _HasConfigMapsChanges(args):
     raise serverless_exceptions.ConfigurationError(
@@ -1363,8 +1501,8 @@ def VerifyManagedFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_GKE]))
 
-  if FlagIsExplicitlySet(
-      args, 'use_http2') and release_track == base.ReleaseTrack.GA:
+  if FlagIsExplicitlySet(args,
+                         'use_http2') and release_track == base.ReleaseTrack.GA:
     raise serverless_exceptions.ConfigurationError(
         'The `--use-http2` flag is only supported in the beta release '
         'track on the fully managed version of Cloud Run. Use `gcloud alpha` '
@@ -1404,15 +1542,6 @@ def VerifyManagedFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_KUBERNETES]))
 
-  if (FlagIsExplicitlySet(args, 'min_instances') and
-      release_track == base.ReleaseTrack.GA):
-    raise serverless_exceptions.ConfigurationError(
-        'The `--min-instances` flag is not supported in the GA release track '
-        'on the fully managed version of Cloud Run. Use `gcloud beta` to set'
-        ' `--min-instances` on Cloud Run (fully managed). Alternatively, '
-        'specify `--platform kubernetes` or run `gcloud config set run/platform'
-        ' kubernetes` to work with Cloud Run for Anthos deployed on VMware.')
-
   if (FlagIsExplicitlySet(args, 'timeout') and
       release_track == base.ReleaseTrack.GA):
     if args.timeout > _FIFTEEN_MINUTES:
@@ -1458,6 +1587,14 @@ def VerifyGKEFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_MANAGED]))
 
+  if FlagIsExplicitlySet(args, 'sandbox'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--sandbox',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
   if FlagIsExplicitlySet(args, 'vpc_connector'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
@@ -1482,6 +1619,30 @@ def VerifyGKEFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_MANAGED]))
 
+  if FlagIsExplicitlySet(args, 'binary_authorization'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--binary-authorization',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
+  if FlagIsExplicitlySet(args, 'clear_binary_authorization'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--clear-binary-authorization',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
+  if FlagIsExplicitlySet(args, 'breakglass'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--breakglass',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
   if FlagIsExplicitlySet(args, 'kubeconfig'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
@@ -1528,6 +1689,14 @@ def VerifyKubernetesFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_MANAGED]))
 
+  if FlagIsExplicitlySet(args, 'sandbox'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--sandbox',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
   if FlagIsExplicitlySet(args, 'vpc_connector'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
@@ -1551,6 +1720,30 @@ def VerifyKubernetesFlags(args, release_track, product):
             platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
                 platforms.PLATFORM_MANAGED]))
 
+  if FlagIsExplicitlySet(args, 'binary_authorization'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--binary-authorization',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
+  if FlagIsExplicitlySet(args, 'clear_binary_authorization'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--clear-binary-authorization',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
+  if FlagIsExplicitlySet(args, 'breakglass'):
+    raise serverless_exceptions.ConfigurationError(
+        error_msg.format(
+            flag='--breakglass',
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
+
   if FlagIsExplicitlySet(args, 'cluster'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
