@@ -817,11 +817,22 @@ def ParseAndSubstituteSSHFlags(args, remote, instance_address,
   if args.ssh_flag:
     for flag in args.ssh_flag:
       for flag_part in flag.split():  # We want grouping here
-        dereferenced_flag = (
-            flag_part.replace('%USER%', remote.user)
-            .replace('%INSTANCE%', instance_address)
-            .replace('%INTERNAL%', internal_address))
-        extra_flags.append(dereferenced_flag)
+
+        deref_flag = flag_part
+
+        # Call replace only when it is necessary. It will lower the chance
+        # replace crashes because the replacing value is None.
+
+        if '%USER%' in deref_flag:
+          deref_flag = deref_flag.replace('%USER%', remote.user)
+
+        if '%INSTANCE%' in deref_flag:
+          deref_flag = deref_flag.replace('%INSTANCE%', instance_address)
+
+        if '%INTERNAL%' in deref_flag:
+          deref_flag = deref_flag.replace('%INTERNAL%', internal_address)
+
+        extra_flags.append(deref_flag)
   return extra_flags
 
 
@@ -1419,6 +1430,12 @@ class SCPCommand(object):
     if self.compress:
       args.append('-C')
 
+    if env.suite is Suite.PUTTY and self.port is None:
+      # TODO(b/178487155) - This addresses a regression in pscp.exe 0.74 in
+      # which the port defaults to 0. It's fixed in the latest dev builds so we
+      # can remove this special handling once 0.75 is released and we bundle
+      # that version.
+      args.extend(['-P', '22'])
     if self.port:
       args.extend(['-P', self.port])
 
