@@ -34,6 +34,7 @@ from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import files
+import six
 
 SERVICE_ACCOUNT_KEY_FILE_FLAG = '--service-account-key-file'
 DOCKER_CREDENTIAL_FILE_FLAG = '--docker-credential-file'
@@ -221,75 +222,76 @@ class Register(base.CreateCommand):
          """),
     )
 
-    if cls.ReleaseTrack() is not base.ReleaseTrack.GA:
-      # Optional groups with required arguments are "modal,"
-      # meaning that if any of the required arguments is specified,
-      # all are required.
-      workload_identity = credentials.add_group(help='Workload Identity')
-      workload_identity.add_argument(
-          '--enable-workload-identity',
-          required=True,
+    # Optional groups with required arguments are "modal,"
+    # meaning that if any of the required arguments is specified,
+    # all are required.
+    workload_identity = credentials.add_group(
+        help='Workload Identity')
+    workload_identity.add_argument(
+        '--enable-workload-identity',
+        required=True,
+        action='store_true',
+        help=textwrap.dedent("""\
+          Enable Workload Identity when registering the cluster with Hub.
+          Requires gcloud alpha or beta.
+          --service_account_key_file flag should not be set if this is set.
+          """),
+    )
+    workload_identity_mutex = workload_identity.add_group(mutex=True)
+    workload_identity_mutex.add_argument(
+        '--public-issuer-url',
+        type=str,
+        help=textwrap.dedent("""\
+          Skip auto-discovery and register the cluster with this issuer URL.
+          Use this option when the OpenID Provider Configuration and associated
+          JSON Web Key Set for validating the cluster's service account JWTs
+          are served at a public endpoint different from the cluster API server.
+          Requires gcloud alpha or beta and --enable-workload-identity.
+          """),
+    )
+    # Keep this hidden as it is not used for user-facing workflows and is
+    # eliminated in beta.
+    if cls.ReleaseTrack() is base.ReleaseTrack.ALPHA:
+      workload_identity_mutex.add_argument(
+          '--manage-workload-identity-bucket',
+          hidden=True,
           action='store_true',
           help=textwrap.dedent("""\
-            Enable Workload Identity when registering the cluster with Hub.
-            Requires gcloud alpha or beta.
-            --service_account_key_file flag should not be set if this is set.
+            Create the GCS bucket for serving OIDC discovery information when
+            registering the cluster with Hub. The cluster must already be
+            configured with an issuer URL of the format:
+            https://storage.googleapis.com/gke-issuer-{UUID}. The cluster must
+            also serve the built-in OIDC discovery endpoints by enabling and
+            correctly configuring the ServiceAccountIssuerDiscovery feature.
+            Requires gcloud alpha and --enable-workload-identity.
+            Mutually exclusive with --public-issuer-url.
             """),
       )
-      workload_identity_mutex = workload_identity.add_group(mutex=True)
+    if cls.ReleaseTrack() is not base.ReleaseTrack.GA:
       workload_identity_mutex.add_argument(
-          '--public-issuer-url',
-          type=str,
+          '--has-private-issuer',
+          hidden=True,
+          action='store_true',
           help=textwrap.dedent("""\
-            Skip auto-discovery and register the cluster with this issuer URL.
-            Use this option when the OpenID Provider Configuration and associated
-            JSON Web Key Set for validating the cluster's service account JWTs
-            are served at a public endpoint different from the cluster API server.
-            Requires gcloud alpha or beta and --enable-workload-identity.
+            Set to true for clusters where no publicly-routable OIDC discovery
+            endpoint for the Kubernetes service account token issuer exists.
+
+            When set to true, the gcloud command-line tool will read the
+            private issuer URL and JSON Web Key Set (JWKS) (public keys) for
+            validating service account tokens from the cluster's API server
+            and upload both when creating the Membership. GCP will then use
+            the JWKS, instead of a public OIDC endpoint, to validate service
+            account tokens issued by this cluster. Note the JWKS establishes
+            the uniqueness of issuers in this configuration, but issuer claims
+            in tokens are still compared to the issuer URL associated with the
+            Membership when validating tokens.
+
+            Note the cluster's OIDC discovery endpoints
+            (https://[KUBE-API-ADDRESS]/.well-known/openid-configuration and
+            https://[KUBE-API-ADDRESS]/openid/v1/jwks) must still be
+            network-accessible to the gcloud client running this command.
             """),
       )
-      # Keep this hidden as it is not used for user-facing workflows and is
-      # eliminated in beta.
-      if cls.ReleaseTrack() is base.ReleaseTrack.ALPHA:
-        workload_identity_mutex.add_argument(
-            '--manage-workload-identity-bucket',
-            hidden=True,
-            action='store_true',
-            help=textwrap.dedent("""\
-              Create the GCS bucket for serving OIDC discovery information when
-              registering the cluster with Hub. The cluster must already be
-              configured with an issuer URL of the format:
-              https://storage.googleapis.com/gke-issuer-{UUID}. The cluster must
-              also serve the built-in OIDC discovery endpoints by enabling and
-              correctly configuring the ServiceAccountIssuerDiscovery feature.
-              Requires gcloud alpha and --enable-workload-identity.
-              Mutually exclusive with --public-issuer-url.
-              """),
-        )
-        workload_identity_mutex.add_argument(
-            '--has-private-issuer',
-            hidden=True,
-            action='store_true',
-            help=textwrap.dedent("""\
-              Set to true for clusters where no publicly-routable OIDC discovery
-              endpoint for the Kubernetes service account token issuer exists.
-
-              When set to true, gcloud will read the private issuer URL and
-              JSON Web Key Set (JWKS) (public keys) for validating service
-              account tokens from the cluster's API server and upload both when
-              creating the Membership. GCP will then use the JWKS, instead of a
-              public OIDC endpoint, to validate service account tokens issued by
-              this cluster. Note the JWKS establishes the uniqueness of issuers
-              in this configuration, but issuer claims in tokens are still
-              compared to the issuer URL associated with the Membership when
-              validating tokens.
-
-              Note the cluster's OIDC discovery endpoints
-              (https://[KUBE-API-ADDRESS]/.well-known/openid-configuration and
-              https://[KUBE-API-ADDRESS]/openid/v1/jwks) must still be
-              network-accessible to the gcloud client running this command.
-              """),
-        )
 
   def Run(self, args):
     project = arg_utils.GetFromNamespace(args, '--project', use_defaults=True)
@@ -325,19 +327,17 @@ class Register(base.CreateCommand):
 
       gke_cluster_self_link = kube_client.processor.gke_cluster_self_link
       issuer_url = None
-      # enable_workload_identity, public_issuer_url, and
-      # manage_workload_identity_bucket are only properties if we are on the
-      # alpha or beta track
-      if (self.ReleaseTrack() is not base.ReleaseTrack.GA
-          and args.enable_workload_identity):
+      private_keyset_json = None
+      if args.enable_workload_identity:
         # public_issuer_url can be None or given by user or gke_cluster_uri
         # (incase of a gke cluster).
         # args.public_issuer_url takes precedence over gke_cluster_uri.
         public_issuer_url = args.public_issuer_url or kube_client.processor.gke_cluster_uri or None
 
         try:
-          openid_config_json = kube_client.GetOpenIDConfiguration(
-              issuer_url=public_issuer_url)
+          openid_config_json = six.ensure_str(
+              kube_client.GetOpenIDConfiguration(issuer_url=public_issuer_url),
+              encoding='utf-8')
         except Exception as e:  # pylint: disable=broad-except
           raise exceptions.Error(
               'Error getting the OpenID Provider Configuration: '
@@ -356,13 +356,22 @@ class Register(base.CreateCommand):
                                  'returned in discovery doc: {}'.format(
                                      public_issuer_url, issuer_url))
 
-        # Set up the GCS bucket that serves OpenID Provider Config and JWKS.
-        if self.ReleaseTrack(
-        ) is base.ReleaseTrack.ALPHA and args.manage_workload_identity_bucket:
-          openid_keyset_json = kube_client.GetOpenIDKeyset()
-          api_util.CreateWorkloadIdentityBucket(project, issuer_url,
-                                                openid_config_json,
-                                                openid_keyset_json)
+        # Request the JWKS from the cluster if we need it (either for setting
+        # up the GCS bucket or getting public keys for private issuers).
+        if self.ReleaseTrack() is base.ReleaseTrack.ALPHA:
+          if args.manage_workload_identity_bucket:
+            api_util.CreateWorkloadIdentityBucket(project, issuer_url,
+                                                  openid_config_json,
+                                                  kube_client.GetOpenIDKeyset())
+        # In the private issuer case(which is supported on ALPHA & BETA),
+        # we set private_keyset_json, which is used later to upload the JWKS
+        # in the Hub Membership. For ALPHA only one of
+        # manage_workload_identity_bucket and has_private_issuer will be set,
+        # since they are mutually exclusive, marked as mutex flags in
+        # surface spec.
+        if self.ReleaseTrack() is not base.ReleaseTrack.GA:
+          if args.has_private_issuer:
+            private_keyset_json = kube_client.GetOpenIDKeyset()
 
       # Attempt to create a membership.
       already_exists = False
@@ -388,7 +397,7 @@ class Register(base.CreateCommand):
                                           args.CLUSTER_NAME,
                                           gke_cluster_self_link, uuid,
                                           self.ReleaseTrack(),
-                                          issuer_url)
+                                          issuer_url, private_keyset_json)
         except apitools_exceptions.HttpConflictError as e:
           # If the error is not due to the object already existing, re-raise.
           error = core_api_exceptions.HttpErrorPayload(e)
@@ -420,20 +429,38 @@ class Register(base.CreateCommand):
       # In case of an existing membership, check with the user to upgrade the
       # Connect-Agent.
       if already_exists:
-        # Update Membership if issuer is updated by the user from an empty value
-        # to a non-empty value or vice versa. UpdateMembership API will error
-        # out if the user tries to modify the issuer URL.
-        if self.ReleaseTrack() is not base.ReleaseTrack.GA and (
+        # Update Membership when required. Scenarios that require updates:
+        # 1. membership.authority is set, but there is now no issuer URL.
+        #    This means the user is disabling Workload Identity.
+        # 2. membership.authority is not set, but there is now an
+        #    issuer URL. This means the user is enabling Workload Identity.
+        # 3. membership.authority is set, but the issuer URL is different
+        #    from that set in membership.authority.issuer. This is technically
+        #    an error, but we defer to validation in the API.
+        # 4. membership.authority.oidcJwks is set, but the private keyset
+        #    we got from the cluster differs from the keyset in the membership.
+        #    This means the user is updating the public keys, and we should
+        #    update to the latest keyset in the membership.
+        if (  # scenario 1, disabling WI
             (obj.authority and not issuer_url) or
+            # scenario 2, enabling WI
             (issuer_url and not obj.authority) or
-            (obj.authority and (obj.authority.issuer != issuer_url))):
+            (obj.authority and
+             # scenario 3, issuer changed
+             ((obj.authority.issuer != issuer_url) or
+              # scenario 4, JWKS changed
+              (private_keyset_json and obj.authority.oidcJwks and
+               (obj.authority.oidcJwks.decode('utf-8') !=
+                private_keyset_json))))):
           console_io.PromptContinue(
               message=hub_util.GenerateWIUpdateMsgString(
                   obj, issuer_url, resource_name, args.CLUSTER_NAME),
               cancel_on_no=True)
           try:
             api_util.UpdateMembership(resource_name, obj, 'authority',
-                                      self.ReleaseTrack(), issuer_url)
+                                      self.ReleaseTrack(),
+                                      issuer_url=issuer_url,
+                                      oidc_jwks=private_keyset_json)
             log.status.Print(
                 'Updated the membership [{}] for the cluster [{}]'.format(
                     resource_name, args.CLUSTER_NAME))
