@@ -23,6 +23,7 @@ import enum
 import os.path
 import uuid
 
+from googlecloudsdk.api_lib.run import k8s_object
 from googlecloudsdk.api_lib.run import traffic
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions as c_exceptions
@@ -109,7 +110,7 @@ class Deploy(base.Command):
     flags.AddAllowUnauthenticatedFlag(managed_group)
     flags.AddCloudSQLFlags(managed_group)
     flags.AddRevisionSuffixArg(managed_group)
-    flags.AddVpcConnectorArg(managed_group)
+    flags.AddVpcConnectorArgs(managed_group)
     flags.AddEgressSettingsFlag(managed_group)
 
     # Flags specific to connecting to a cluster
@@ -132,6 +133,7 @@ class Deploy(base.Command):
     flags.AddTimeoutFlag(parser)
     flags.AddAsyncFlag(parser)
     flags.AddLabelsFlags(parser)
+    flags.AddMinInstancesFlag(parser)
     flags.AddMaxInstancesFlag(parser)
     flags.AddCommandFlag(parser)
     flags.AddArgsFlag(parser)
@@ -151,14 +153,14 @@ class Deploy(base.Command):
 
     # Flags only supported on GKE and Knative
     cluster_group = flags.GetClusterArgGroup(parser)
-    flags.AddMinInstancesFlag(cluster_group)
     flags.AddEndpointVisibilityEnum(cluster_group)
     flags.AddHttp2Flag(cluster_group)
 
   def Run(self, args):
     """Deploy a container to Cloud Run."""
     flags.GetAndValidatePlatform(args, self.ReleaseTrack(), flags.Product.RUN)
-    service_ref = flags.GetService(args)
+    service_ref = args.CONCEPTS.service.Parse()
+    flags.ValidateResource(service_ref)
     build_type = None
     image = None
     pack = None
@@ -192,6 +194,10 @@ class Deploy(base.Command):
     conn_context = connection_context.GetConnectionContext(
         args, flags.Product.RUN, self.ReleaseTrack())
     changes = flags.GetConfigurationChanges(args)
+    changes.insert(
+        0,
+        config_changes.DeleteAnnotationChange(
+            k8s_object.BINAUTHZ_BREAKGLASS_ANNOTATION))
     changes.append(
         config_changes.SetLaunchStageAnnotationChange(self.ReleaseTrack()))
 
@@ -257,7 +263,6 @@ class BetaDeploy(Deploy):
     flags.AddEndpointVisibilityEnum(cluster_group)
 
     # Flags not specific to any platform
-    flags.AddMinInstancesFlag(parser)
     flags.AddDeployTagFlag(parser)
     flags.AddIngressFlag(parser)
     flags.AddHttp2Flag(parser)
@@ -275,12 +280,17 @@ class AlphaDeploy(Deploy):
   def Args(parser):
     Deploy.CommonArgs(parser)
 
+    # Flags specific to managed CR
+    managed_group = flags.GetManagedArgGroup(parser)
+    flags.AddSandboxArg(managed_group)
+    flags.AddBinAuthzPolicyFlags(managed_group)
+    flags.AddBinAuthzBreakglassFlag(managed_group)
+
     # Flags specific to connecting to a cluster
     cluster_group = flags.GetClusterArgGroup(parser)
     flags.AddEndpointVisibilityEnum(cluster_group, deprecated=True)
 
     # Flags not specific to any platform
-    flags.AddMinInstancesFlag(parser)
     flags.AddDeployTagFlag(parser)
     flags.AddIngressFlag(parser)
     flags.AddHttp2Flag(parser)
