@@ -29,9 +29,11 @@ import sys
 import time
 
 import gslib
+from gslib.commands import ls
 from gslib.cs_api_map import ApiSelector
 from gslib.project_id import PopulateProjectId
 import gslib.tests.testcase as testcase
+from gslib.tests.testcase.integration_testcase import SkipForGS
 from gslib.tests.testcase.integration_testcase import SkipForS3
 from gslib.tests.testcase.integration_testcase import SkipForXML
 from gslib.tests.util import CaptureStdout
@@ -154,6 +156,21 @@ class TestLsUnit(testcase.GsUtilUnitTestCase):
         stor_update_time,
         datetime.strftime(new_update_time, '%a, %d %b %Y %H:%M:%S GMT'))
 
+  @mock.patch.object(ls.LsCommand, 'WildcardIterator')
+  def test_satisfies_pzs_is_displayed_if_present(self, mock_wildcard):
+    bucket_uri = self.CreateBucket(bucket_name='foo')
+    bucket_metadata = apitools_messages.Bucket(name='foo', satisfiesPZS=True)
+    bucket_uri.root_object = bucket_metadata
+    bucket_uri.url_string = 'foo'
+    bucket_uri.storage_url = mock.Mock()
+
+    mock_wildcard.return_value.IterBuckets.return_value = [bucket_uri]
+    # MockKey doesn't support hash_algs, so the MD5 will not match.
+    with SetBotoConfigForTest([('GSUtil', 'check_hashes', 'never')]):
+      stdout = self.RunCommand('ls', ['-Lb', suri(bucket_uri)],
+                               return_stdout=True)
+    self.assertRegex(stdout, 'Satisfies PZS:\t\t\tTrue')
+
 
 class TestLs(testcase.GsUtilIntegrationTestCase):
   """Integration tests for ls command."""
@@ -316,6 +333,34 @@ class TestLs(testcase.GsUtilIntegrationTestCase):
 
     _Check1()
 
+  def location_redirect_test_helper(self, bucket_region, client_region):
+    bucket_host = 's3.%s.amazonaws.com' % bucket_region
+    client_host = 's3.%s.amazonaws.com' % client_region
+    with SetBotoConfigForTest([('s3', 'host', bucket_host)]):
+      bucket_uri = self.CreateBucket(location=bucket_region)
+      obj_uri = self.CreateObject(bucket_uri=bucket_uri, contents=b'foo')
+
+    @Retry(AssertionError, tries=3, timeout_secs=1)
+    def _Check1(uri):
+      stdout = self.RunGsUtil(['ls', uri], return_stdout=True)
+      self.assertEqual('%s\n' % obj_uri, stdout)
+
+    with SetBotoConfigForTest([('s3', 'host', client_host)]):
+      # sends a GET request
+      _Check1(suri(bucket_uri))
+      # sends a HEAD request, meaning error body is not included.
+      _Check1(suri(obj_uri))
+
+  @SkipForGS('Only s3 V4 signatures error on location mismatches.')
+  def test_400_location_redirect(self):
+    # ap-east-1 used here since regions launched before March 20, 2019 do
+    # some temporary redirecting for new buckets which suppresses 400 errors.
+    self.location_redirect_test_helper('ap-east-1', 'us-east-2')
+
+  @SkipForGS('Only s3 V4 signatures error on location mismatches.')
+  def test_301_location_redirect(self):
+    self.location_redirect_test_helper('eu-west-1', 'us-east-2')
+
   @SkipForXML('Credstore file gets created only for json API')
   def test_credfile_lock_permissions(self):
     tmpdir = self.CreateTempDir()
