@@ -36,9 +36,9 @@
   Other compilers, like gcc, optimize these dependencies by default.
 
   Since the instrumented APIs declared here are wrapper on top
-  of my_pthread / safemutex / etc APIs,
+  of my_thread / safemutex / etc APIs,
   including mysql/psi/mysql_thread.h assumes that
-  the dependency on my_pthread and safemutex already exists.
+  the dependency on my_thread and safemutex already exists.
 */
 /*
   Note: there are several orthogonal dimensions here.
@@ -56,12 +56,37 @@
   - the pthread library
   - fast mutexes
   - window apis
-  This is implemented by various macro definitions in my_pthread.h
+  This is implemented by various macro definitions in my_thread.h
 
   This causes complexity with '#ifdef'-ery that can't be avoided.
 */
 
+#include "my_thread.h"
+#include "my_thread_local.h"
+#include "thr_mutex.h"
+#include "thr_rwlock.h"
 #include "mysql/psi/psi.h"
+#ifdef MYSQL_SERVER
+#ifndef MYSQL_DYNAMIC_PLUGIN
+#include "pfs_thread_provider.h"
+#endif
+#endif
+
+#ifndef PSI_MUTEX_CALL
+#define PSI_MUTEX_CALL(M) PSI_DYNAMIC_CALL(M)
+#endif
+
+#ifndef PSI_RWLOCK_CALL
+#define PSI_RWLOCK_CALL(M) PSI_DYNAMIC_CALL(M)
+#endif
+
+#ifndef PSI_COND_CALL
+#define PSI_COND_CALL(M) PSI_DYNAMIC_CALL(M)
+#endif
+
+#ifndef PSI_THREAD_CALL
+#define PSI_THREAD_CALL(M) PSI_DYNAMIC_CALL(M)
+#endif
 
 /**
   @defgroup Thread_instrumentation Thread Instrumentation
@@ -76,13 +101,7 @@
 struct st_mysql_mutex
 {
   /** The real mutex. */
-#ifdef SAFE_MUTEX
-  safe_mutex_t m_mutex;
-#elif defined(MY_PTHREAD_FASTMUTEX)
-  my_pthread_fastmutex_t m_mutex;
-#else
-  pthread_mutex_t m_mutex;
-#endif
+  my_mutex_t m_mutex;
   /**
     The instrumentation hook.
     Note that this hook is not conditionally defined,
@@ -93,7 +112,7 @@ struct st_mysql_mutex
 
 /**
   Type of an instrumented mutex.
-  @c mysql_mutex_t is a drop-in replacement for @c pthread_mutex_t.
+  @c mysql_mutex_t is a drop-in replacement for @c my_mutex_t.
   @sa mysql_mutex_assert_owner
   @sa mysql_mutex_assert_not_owner
   @sa mysql_mutex_init
@@ -110,7 +129,7 @@ typedef struct st_mysql_mutex mysql_mutex_t;
 struct st_mysql_rwlock
 {
   /** The real rwlock */
-  rw_lock_t m_rwlock;
+  native_rw_lock_t m_rwlock;
   /**
     The instrumentation hook.
     Note that this hook is not conditionally defined,
@@ -167,7 +186,7 @@ typedef struct st_mysql_prlock mysql_prlock_t;
 struct st_mysql_cond
 {
   /** The real condition */
-  pthread_cond_t m_cond;
+  native_cond_t m_cond;
   /**
     The instrumentation hook.
     Note that this hook is not conditionally defined,
@@ -178,7 +197,7 @@ struct st_mysql_cond
 
 /**
   Type of an instrumented condition.
-  @c mysql_cond_t is a drop-in replacement for @c pthread_cond_t.
+  @c mysql_cond_t is a drop-in replacement for @c native_cond_t.
   @sa mysql_cond_init
   @sa mysql_cond_wait
   @sa mysql_cond_timedwait
@@ -214,8 +233,12 @@ typedef struct st_mysql_cond mysql_cond_t;
   @c mysql_mutex_assert_owner is a drop-in replacement
   for @c safe_mutex_assert_owner.
 */
+#ifdef SAFE_MUTEX
 #define mysql_mutex_assert_owner(M) \
-  safe_mutex_assert_owner(&(M)->m_mutex)
+  safe_mutex_assert_owner(&(M)->m_mutex);
+#else
+#define mysql_mutex_assert_owner(M) { }
+#endif
 
 /**
   @def mysql_mutex_assert_not_owner(M)
@@ -223,14 +246,26 @@ typedef struct st_mysql_cond mysql_cond_t;
   @c mysql_mutex_assert_not_owner is a drop-in replacement
   for @c safe_mutex_assert_not_owner.
 */
+#ifdef SAFE_MUTEX
 #define mysql_mutex_assert_not_owner(M) \
-  safe_mutex_assert_not_owner(&(M)->m_mutex)
-
-/** Wrappers for instrumented prlock objects.  */
+  safe_mutex_assert_not_owner(&(M)->m_mutex);
+#else
+#define mysql_mutex_assert_not_owner(M) { }
+#endif
 
+/**
+  @def mysql_prlock_assert_write_owner(M)
+  Drop-in replacement
+  for @c rw_pr_lock_assert_write_owner.
+*/
 #define mysql_prlock_assert_write_owner(M) \
   rw_pr_lock_assert_write_owner(&(M)->m_prlock)
 
+/**
+  @def mysql_prlock_assert_not_write_owner(M)
+  Drop-in replacement
+  for @c rw_pr_lock_assert_not_write_owner.
+*/
 #define mysql_prlock_assert_not_write_owner(M) \
   rw_pr_lock_assert_not_write_owner(&(M)->m_prlock)
 
@@ -301,7 +336,7 @@ typedef struct st_mysql_cond mysql_cond_t;
   @def mysql_mutex_trylock(M)
   Instrumented mutex_lock.
   @c mysql_mutex_trylock is a drop-in replacement
-  for @c pthread_mutex_trylock.
+  for @c my_mutex_trylock.
 */
 
 #if defined(SAFE_MUTEX) || defined (HAVE_PSI_MUTEX_INTERFACE)
@@ -483,17 +518,18 @@ typedef struct st_mysql_cond mysql_cond_t;
   inline_mysql_cond_register(P1, P2, P3)
 
 /**
-  @def mysql_cond_init(K, C, A)
+  @def mysql_cond_init(K, C)
   Instrumented cond_init.
   @c mysql_cond_init is a replacement for @c pthread_cond_init.
+  Note that pthread_condattr_t is not supported in MySQL.
   @param C The cond to initialize
   @param K The PSI_cond_key for this instrumented cond
-  @param A Condition attributes
+
 */
 #ifdef HAVE_PSI_COND_INTERFACE
-  #define mysql_cond_init(K, C, A) inline_mysql_cond_init(K, C, A)
+  #define mysql_cond_init(K, C) inline_mysql_cond_init(K, C)
 #else
-  #define mysql_cond_init(K, C, A) inline_mysql_cond_init(C, A)
+  #define mysql_cond_init(K, C) inline_mysql_cond_init(C)
 #endif
 
 /**
@@ -506,9 +542,9 @@ typedef struct st_mysql_cond mysql_cond_t;
 /**
   @def mysql_cond_wait(C)
   Instrumented cond_wait.
-  @c mysql_cond_wait is a drop-in replacement for @c pthread_cond_wait.
+  @c mysql_cond_wait is a drop-in replacement for @c native_cond_wait.
 */
-#ifdef HAVE_PSI_COND_INTERFACE
+#if defined(SAFE_MUTEX) || defined(HAVE_PSI_COND_INTERFACE)
   #define mysql_cond_wait(C, M) \
     inline_mysql_cond_wait(C, M, __FILE__, __LINE__)
 #else
@@ -520,9 +556,9 @@ typedef struct st_mysql_cond mysql_cond_t;
   @def mysql_cond_timedwait(C, M, W)
   Instrumented cond_timedwait.
   @c mysql_cond_timedwait is a drop-in replacement
-  for @c pthread_cond_timedwait.
+  for @c native_cond_timedwait.
 */
-#ifdef HAVE_PSI_COND_INTERFACE
+#if defined(SAFE_MUTEX) || defined(HAVE_PSI_COND_INTERFACE)
   #define mysql_cond_timedwait(C, M, W) \
     inline_mysql_cond_timedwait(C, M, W, __FILE__, __LINE__)
 #else
@@ -554,9 +590,9 @@ typedef struct st_mysql_cond mysql_cond_t;
 
 /**
   @def mysql_thread_create(K, P1, P2, P3, P4)
-  Instrumented pthread_create.
+  Instrumented my_thread_create.
   This function creates both the thread instrumentation and a thread.
-  @c mysql_thread_create is a replacement for @c pthread_create.
+  @c mysql_thread_create is a replacement for @c my_thread_create.
   The parameter P4 (or, if it is NULL, P1) will be used as the
   instrumented thread "indentity".
   Providing a P1 / P4 parameter with a different value for each call
@@ -564,22 +600,22 @@ typedef struct st_mysql_cond mysql_cond_t;
   is used internally to randomize access to data and prevent contention.
   This is optional, and the improvement is not guaranteed, only statistical.
   @param K The PSI_thread_key for this instrumented thread
-  @param P1 pthread_create parameter 1
-  @param P2 pthread_create parameter 2
-  @param P3 pthread_create parameter 3
-  @param P4 pthread_create parameter 4
+  @param P1 my_thread_create parameter 1
+  @param P2 my_thread_create parameter 2
+  @param P3 my_thread_create parameter 3
+  @param P4 my_thread_create parameter 4
 */
 #ifdef HAVE_PSI_THREAD_INTERFACE
   #define mysql_thread_create(K, P1, P2, P3, P4) \
     inline_mysql_thread_create(K, P1, P2, P3, P4)
 #else
   #define mysql_thread_create(K, P1, P2, P3, P4) \
-    pthread_create(P1, P2, P3, P4)
+    my_thread_create(P1, P2, P3, P4)
 #endif
 
 /**
   @def mysql_thread_set_psi_id(I)
-  Set the thread indentifier for the instrumentation.
+  Set the thread identifier for the instrumentation.
   @param I The thread identifier
 */
 #ifdef HAVE_PSI_THREAD_INTERFACE
@@ -588,6 +624,17 @@ typedef struct st_mysql_cond mysql_cond_t;
   #define mysql_thread_set_psi_id(I) do {} while (0)
 #endif
 
+/**
+  @def mysql_thread_set_psi_THD(T)
+  Set the thread sql session for the instrumentation.
+  @param I The thread identifier
+*/
+#ifdef HAVE_PSI_THREAD_INTERFACE
+  #define mysql_thread_set_psi_THD(T) inline_mysql_thread_set_psi_THD(T)
+#else
+  #define mysql_thread_set_psi_THD(T) do {} while (0)
+#endif
+
 static inline void inline_mysql_mutex_register(
 #ifdef HAVE_PSI_MUTEX_INTERFACE
   const char *category,
@@ -610,7 +657,7 @@ static inline int inline_mysql_mutex_init(
   PSI_mutex_key key,
 #endif
   mysql_mutex_t *that,
-  const pthread_mutexattr_t *attr
+  const native_mutexattr_t *attr
 #ifdef SAFE_MUTEX
   , const char *src_file, uint src_line
 #endif
@@ -621,13 +668,11 @@ static inline int inline_mysql_mutex_init(
 #else
   that->m_psi= NULL;
 #endif
+  return my_mutex_init(&that->m_mutex, attr
 #ifdef SAFE_MUTEX
-  return safe_mutex_init(&that->m_mutex, attr, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-  return my_pthread_fastmutex_init(&that->m_mutex, attr);
-#else
-  return pthread_mutex_init(&that->m_mutex, attr);
+                       , src_file, src_line
 #endif
+                       );
 }
 
 static inline int inline_mysql_mutex_destroy(
@@ -644,13 +689,11 @@ static inline int inline_mysql_mutex_destroy(
     that->m_psi= NULL;
   }
 #endif
+  return my_mutex_destroy(&that->m_mutex
 #ifdef SAFE_MUTEX
-  return safe_mutex_destroy(&that->m_mutex, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-  return pthread_mutex_destroy(&that->m_mutex.mutex);
-#else
-  return pthread_mutex_destroy(&that->m_mutex);
+                          , src_file, src_line
 #endif
+                          );
 }
 
 static inline int inline_mysql_mutex_lock(
@@ -672,13 +715,11 @@ static inline int inline_mysql_mutex_lock(
                                        PSI_MUTEX_LOCK, src_file, src_line);
 
     /* Instrumented code */
+    result= my_mutex_lock(&that->m_mutex
 #ifdef SAFE_MUTEX
-    result= safe_mutex_lock(&that->m_mutex, FALSE, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-    result= my_pthread_fastmutex_lock(&that->m_mutex);
-#else
-    result= pthread_mutex_lock(&that->m_mutex);
+                          , src_file, src_line
 #endif
+                          );
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -689,13 +730,11 @@ static inline int inline_mysql_mutex_lock(
 #endif
 
   /* Non instrumented code */
+  result= my_mutex_lock(&that->m_mutex
 #ifdef SAFE_MUTEX
-  result= safe_mutex_lock(&that->m_mutex, FALSE, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-  result= my_pthread_fastmutex_lock(&that->m_mutex);
-#else
-  result= pthread_mutex_lock(&that->m_mutex);
+                        , src_file, src_line
 #endif
+                        );
 
   return result;
 }
@@ -719,13 +758,11 @@ static inline int inline_mysql_mutex_trylock(
                                        PSI_MUTEX_TRYLOCK, src_file, src_line);
 
     /* Instrumented code */
+    result= my_mutex_trylock(&that->m_mutex
 #ifdef SAFE_MUTEX
-    result= safe_mutex_lock(&that->m_mutex, TRUE, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-    result= pthread_mutex_trylock(&that->m_mutex.mutex);
-#else
-    result= pthread_mutex_trylock(&that->m_mutex);
+                             , src_file, src_line
 #endif
+                             );
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -736,13 +773,11 @@ static inline int inline_mysql_mutex_trylock(
 #endif
 
   /* Non instrumented code */
+  result= my_mutex_trylock(&that->m_mutex
 #ifdef SAFE_MUTEX
-  result= safe_mutex_lock(&that->m_mutex, TRUE, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-  result= pthread_mutex_trylock(&that->m_mutex.mutex);
-#else
-  result= pthread_mutex_trylock(&that->m_mutex);
+                           , src_file, src_line
 #endif
+                           );
 
   return result;
 }
@@ -761,13 +796,11 @@ static inline int inline_mysql_mutex_unlock(
     PSI_MUTEX_CALL(unlock_mutex)(that->m_psi);
 #endif
 
+  result= my_mutex_unlock(&that->m_mutex
 #ifdef SAFE_MUTEX
-  result= safe_mutex_unlock(&that->m_mutex, src_file, src_line);
-#elif defined(MY_PTHREAD_FASTMUTEX)
-  result= pthread_mutex_unlock(&that->m_mutex.mutex);
-#else
-  result= pthread_mutex_unlock(&that->m_mutex);
+                          , src_file, src_line
 #endif
+                          );
 
   return result;
 }
@@ -800,10 +833,7 @@ static inline int inline_mysql_rwlock_init(
 #else
   that->m_psi= NULL;
 #endif
-  /*
-    pthread_rwlockattr_t is not used in MySQL.
-  */
-  return my_rwlock_init(&that->m_rwlock, NULL);
+  return native_rw_init(&that->m_rwlock);
 }
 
 #ifndef DISABLE_MYSQL_PRLOCK_H
@@ -832,7 +862,7 @@ static inline int inline_mysql_rwlock_destroy(
     that->m_psi= NULL;
   }
 #endif
-  return rwlock_destroy(&that->m_rwlock);
+  return native_rw_destroy(&that->m_rwlock);
 }
 
 #ifndef DISABLE_MYSQL_PRLOCK_H
@@ -869,7 +899,7 @@ static inline int inline_mysql_rwlock_rdlock(
                                           PSI_RWLOCK_READLOCK, src_file, src_line);
 
     /* Instrumented code */
-    result= rw_rdlock(&that->m_rwlock);
+    result= native_rw_rdlock(&that->m_rwlock);
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -880,7 +910,7 @@ static inline int inline_mysql_rwlock_rdlock(
 #endif
 
   /* Non instrumented code */
-  result= rw_rdlock(&that->m_rwlock);
+  result= native_rw_rdlock(&that->m_rwlock);
 
   return result;
 }
@@ -941,7 +971,7 @@ static inline int inline_mysql_rwlock_wrlock(
                                           PSI_RWLOCK_WRITELOCK, src_file, src_line);
 
     /* Instrumented code */
-    result= rw_wrlock(&that->m_rwlock);
+    result= native_rw_wrlock(&that->m_rwlock);
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -952,7 +982,7 @@ static inline int inline_mysql_rwlock_wrlock(
 #endif
 
   /* Non instrumented code */
-  result= rw_wrlock(&that->m_rwlock);
+  result= native_rw_wrlock(&that->m_rwlock);
 
   return result;
 }
@@ -1013,7 +1043,7 @@ static inline int inline_mysql_rwlock_tryrdlock(
                                           PSI_RWLOCK_TRYREADLOCK, src_file, src_line);
 
     /* Instrumented code */
-    result= rw_tryrdlock(&that->m_rwlock);
+    result= native_rw_tryrdlock(&that->m_rwlock);
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -1024,7 +1054,7 @@ static inline int inline_mysql_rwlock_tryrdlock(
 #endif
 
   /* Non instrumented code */
-  result= rw_tryrdlock(&that->m_rwlock);
+  result= native_rw_tryrdlock(&that->m_rwlock);
 
   return result;
 }
@@ -1048,7 +1078,7 @@ static inline int inline_mysql_rwlock_trywrlock(
                                           PSI_RWLOCK_TRYWRITELOCK, src_file, src_line);
 
     /* Instrumented code */
-    result= rw_trywrlock(&that->m_rwlock);
+    result= native_rw_trywrlock(&that->m_rwlock);
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -1059,7 +1089,7 @@ static inline int inline_mysql_rwlock_trywrlock(
 #endif
 
   /* Non instrumented code */
-  result= rw_trywrlock(&that->m_rwlock);
+  result= native_rw_trywrlock(&that->m_rwlock);
 
   return result;
 }
@@ -1072,7 +1102,7 @@ static inline int inline_mysql_rwlock_unlock(
   if (that->m_psi != NULL)
     PSI_RWLOCK_CALL(unlock_rwlock)(that->m_psi);
 #endif
-  result= rw_unlock(&that->m_rwlock);
+  result= native_rw_unlock(&that->m_rwlock);
   return result;
 }
 
@@ -1111,15 +1141,14 @@ static inline int inline_mysql_cond_init(
 #ifdef HAVE_PSI_COND_INTERFACE
   PSI_cond_key key,
 #endif
-  mysql_cond_t *that,
-  const pthread_condattr_t *attr)
+  mysql_cond_t *that)
 {
 #ifdef HAVE_PSI_COND_INTERFACE
   that->m_psi= PSI_COND_CALL(init_cond)(key, &that->m_cond);
 #else
   that->m_psi= NULL;
 #endif
-  return pthread_cond_init(&that->m_cond, attr);
+  return native_cond_init(&that->m_cond);
 }
 
 static inline int inline_mysql_cond_destroy(
@@ -1132,13 +1161,13 @@ static inline int inline_mysql_cond_destroy(
     that->m_psi= NULL;
   }
 #endif
-  return pthread_cond_destroy(&that->m_cond);
+  return native_cond_destroy(&that->m_cond);
 }
 
 static inline int inline_mysql_cond_wait(
   mysql_cond_t *that,
   mysql_mutex_t *mutex
-#ifdef HAVE_PSI_COND_INTERFACE
+#if defined(SAFE_MUTEX) || defined(HAVE_PSI_COND_INTERFACE)
   , const char *src_file, uint src_line
 #endif
   )
@@ -1155,7 +1184,11 @@ static inline int inline_mysql_cond_wait(
                                       PSI_COND_WAIT, src_file, src_line);
 
     /* Instrumented code */
-    result= my_cond_wait(&that->m_cond, &mutex->m_mutex);
+    result= my_cond_wait(&that->m_cond, &mutex->m_mutex
+#ifdef SAFE_MUTEX
+                         , src_file, src_line
+#endif
+                         );
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -1166,7 +1199,11 @@ static inline int inline_mysql_cond_wait(
 #endif
 
   /* Non instrumented code */
-  result= my_cond_wait(&that->m_cond, &mutex->m_mutex);
+  result= my_cond_wait(&that->m_cond, &mutex->m_mutex
+#ifdef SAFE_MUTEX
+                       , src_file, src_line
+#endif
+                       );
 
   return result;
 }
@@ -1175,7 +1212,7 @@ static inline int inline_mysql_cond_timedwait(
   mysql_cond_t *that,
   mysql_mutex_t *mutex,
   const struct timespec *abstime
-#ifdef HAVE_PSI_COND_INTERFACE
+#if defined(SAFE_MUTEX) || defined(HAVE_PSI_COND_INTERFACE)
   , const char *src_file, uint src_line
 #endif
   )
@@ -1192,7 +1229,11 @@ static inline int inline_mysql_cond_timedwait(
                                       PSI_COND_TIMEDWAIT, src_file, src_line);
 
     /* Instrumented code */
-    result= my_cond_timedwait(&that->m_cond, &mutex->m_mutex, abstime);
+    result= my_cond_timedwait(&that->m_cond, &mutex->m_mutex, abstime
+#ifdef SAFE_MUTEX
+                              , src_file, src_line
+#endif
+                              );
 
     /* Instrumentation end */
     if (locker != NULL)
@@ -1203,7 +1244,11 @@ static inline int inline_mysql_cond_timedwait(
 #endif
 
   /* Non instrumented code */
-  result= my_cond_timedwait(&that->m_cond, &mutex->m_mutex, abstime);
+  result= my_cond_timedwait(&that->m_cond, &mutex->m_mutex, abstime
+#ifdef SAFE_MUTEX
+                            , src_file, src_line
+#endif
+                            );
 
   return result;
 }
@@ -1216,7 +1261,7 @@ static inline int inline_mysql_cond_signal(
   if (that->m_psi != NULL)
     PSI_COND_CALL(signal_cond)(that->m_psi);
 #endif
-  result= pthread_cond_signal(&that->m_cond);
+  result= native_cond_signal(&that->m_cond);
   return result;
 }
 
@@ -1228,7 +1273,7 @@ static inline int inline_mysql_cond_broadcast(
   if (that->m_psi != NULL)
     PSI_COND_CALL(broadcast_cond)(that->m_psi);
 #endif
-  result= pthread_cond_broadcast(&that->m_cond);
+  result= native_cond_broadcast(&that->m_cond);
   return result;
 }
 
@@ -1252,19 +1297,29 @@ static inline void inline_mysql_thread_register(
 #ifdef HAVE_PSI_THREAD_INTERFACE
 static inline int inline_mysql_thread_create(
   PSI_thread_key key,
-  pthread_t *thread, const pthread_attr_t *attr,
-  void *(*start_routine)(void*), void *arg)
+  my_thread_handle *thread, const my_thread_attr_t *attr,
+  my_start_routine start_routine, void *arg)
 {
   int result;
   result= PSI_THREAD_CALL(spawn_thread)(key, thread, attr, start_routine, arg);
   return result;
 }
 
-static inline void inline_mysql_thread_set_psi_id(ulong id)
+static inline void inline_mysql_thread_set_psi_id(my_thread_id id)
 {
   struct PSI_thread *psi= PSI_THREAD_CALL(get_thread)();
   PSI_THREAD_CALL(set_thread_id)(psi, id);
 }
+
+#ifdef __cplusplus
+class THD;
+static inline void inline_mysql_thread_set_psi_THD(THD *thd)
+{
+  struct PSI_thread *psi= PSI_THREAD_CALL(get_thread)();
+  PSI_THREAD_CALL(set_thread_THD)(psi, thd);
+}
+#endif /* __cplusplus */
+
 #endif
 
 #endif /* DISABLE_MYSQL_THREAD_H */
