@@ -1,7 +1,7 @@
 <?php
 
 /*
- * Copyright (C) 2015 Deciso B.V.
+ * Copyright (C) 2015-2021 Deciso B.V.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,9 @@ namespace OPNsense\Base;
 
 use OPNsense\Core\Config;
 use Phalcon\Mvc\Controller;
+use Phalcon\Logger;
 use Phalcon\Logger\Adapter\Syslog;
+use Phalcon\Translate\InterpolatorFactory;
 use OPNsense\Core\ACL;
 
 /**
@@ -44,6 +46,12 @@ class ControllerRoot extends Controller
      */
     public $translator;
 
+
+    /**
+     * log handle
+     */
+    protected $logger = null;
+
     /**
      * @var null|string logged in username, populated during authentication
      */
@@ -83,12 +91,15 @@ class ControllerRoot extends Controller
         }
 
         $locale = $lang . '.UTF-8';
-        bind_textdomain_codeset('OPNsense', $locale);
-        $this->translator = new ViewTranslator(array(
+        $interpolator = new InterpolatorFactory();
+        $this->translator = new ViewTranslator($interpolator, array(
             'directory' => '/usr/local/share/locale',
             'defaultDomain' => 'OPNsense',
             'locale' => $locale,
         ));
+
+        /* somehow this is not done by Phalcon */
+        putenv('LANG=' . $locale);
     }
 
     /**
@@ -98,12 +109,18 @@ class ControllerRoot extends Controller
      */
     protected function getLogger($ident = "api")
     {
-        $logger = new Syslog($ident, array(
-            'option' => LOG_PID,
-            'facility' => LOG_LOCAL4
-        ));
-
-        return $logger;
+        if ($this->logger == null) {
+            $this->logger = new Logger(
+                'messages',
+                [
+                    'main' => new Syslog($ident, array(
+                        'option' => LOG_PID,
+                        'facility' => LOG_LOCAL4
+                    ))
+                ]
+            );
+        }
+        return $this->logger;
     }
 
     /**
