@@ -29,6 +29,7 @@
 namespace OPNsense\Core;
 
 use Phalcon\DI\FactoryDefault;
+use Phalcon\Logger;
 use Phalcon\Logger\Adapter\Syslog;
 
 /**
@@ -285,7 +286,15 @@ class Config extends Singleton
             $this->simplexml = null;
             // there was an issue with loading the config, try to restore the last backup
             $backups = $this->getBackups();
-            $logger = new Syslog("config", array('option' => LOG_PID, 'facility' => LOG_LOCAL4));
+            $logger = new Logger(
+                'messages',
+                [
+                    'main' => new Syslog("config", array(
+                        'option' => LOG_PID,
+                        'facility' => LOG_LOCAL4
+                    ))
+                ]
+            );
             if (count($backups) > 0) {
                 // load last backup
                 $logger->error(gettext('No valid config.xml found, attempting last known config restore.'));
@@ -454,6 +463,27 @@ class Config extends Singleton
         }
     }
 
+    /**
+     * send config change to audit log including the context we currently know of.
+     */
+    private function auditLogChange($backup_filename, $revision = null)
+    {
+        openlog("audit", LOG_ODELAY, LOG_AUTH);
+        $append_message = "";
+        if (is_array($revision) && !empty($revision['description'])) {
+            $append_message = sprintf(" [%s]", $revision['description']);
+        }
+        syslog(LOG_NOTICE, sprintf(
+            "user %s%s changed configuration to %s in %s%s",
+            !empty($_SESSION["Username"]) ? $_SESSION["Username"] : "(system)",
+            !empty($_SERVER['REMOTE_ADDR']) ? "@" . $_SERVER['REMOTE_ADDR'] : "",
+            $backup_filename,
+            !empty($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : $_SERVER['SCRIPT_NAME'],
+            $append_message
+        ));
+        closelog();
+    }
+
     /**
      * backup current config
      * @return string target filename
@@ -617,10 +647,19 @@ class Config extends Singleton
                 fflush($this->config_file_handle);
                 $backup_filename = $backup ? $this->backup() : null;
                 if ($backup_filename) {
+                    $this->auditLogChange($backup_filename, $revision);
                     // use syslog to trigger a new configd event, which should signal a syshook config (in batch).
                     // Althought we include the backup filename, the event handler is responsible to determine the
                     // last processed event itself. (it's merely added for debug purposes)
-                    $logger = new Syslog("config", array('option' => LOG_PID, 'facility' => LOG_LOCAL5));
+                    $logger = new Logger(
+                        'messages',
+                        [
+                            'main' => new Syslog("config", array(
+                                'option' => LOG_PID,
+                                'facility' => LOG_LOCAL5
+                            ))
+                        ]
+                    );
                     $logger->info("config-event: new_config " . $backup_filename);
                 }
                 flock($this->config_file_handle, LOCK_UN);
