@@ -4,6 +4,7 @@
 Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
+
 from __future__ import print_function
 
 import csv
@@ -22,12 +23,13 @@ from core.settings import config
 from core.settings import BOGON_RANGES
 from core.settings import CHECK_CONNECTION_URL
 from core.settings import CDN_RANGES
-from core.settings import NAME
 from core.settings import IPCAT_SQLITE_FILE
 from core.settings import IS_WIN
+from core.settings import MAX_HELP_OPTION_LENGTH
 from core.settings import STATIC_IPCAT_LOOKUPS
 from core.settings import TIMEOUT
 from core.settings import UNICODE_ENCODING
+from core.settings import USER_AGENT
 from core.settings import WHITELIST
 from core.settings import WHITELIST_RANGES
 from core.settings import WORST_ASNS
@@ -43,7 +45,7 @@ def retrieve_content(url, data=None, headers=None):
     """
 
     try:
-        req = _urllib.request.Request("".join(url[i].replace(' ', "%20") if i > url.find('?') else url[i] for i in xrange(len(url))), data, headers or {"User-agent": NAME, "Accept-encoding": "gzip, deflate"})
+        req = _urllib.request.Request("".join(url[i].replace(' ', "%20") if i > url.find('?') else url[i] for i in xrange(len(url))), data, headers or {"User-agent": USER_AGENT, "Accept-encoding": "gzip, deflate"})
         resp = _urllib.request.urlopen(req, timeout=TIMEOUT)
         retval = resp.read()
         encoding = resp.headers.get("Content-Encoding")
@@ -148,7 +150,7 @@ def bogon_ip(address):
 
 def check_sudo():
     """
-    Checks for sudo/Administrator privileges
+    Checks for root privileges
     """
 
     check = None
@@ -294,3 +296,24 @@ def get_ex_message(ex):
 
 def is_local(address):
     return re.search(r"\A(127|10|172\.[13][0-9]|192\.168)\.", address or "") is not None
+
+def patch_parser(parser):
+    # Dirty hack to display longer options without breaking into two lines
+    if hasattr(parser, "formatter"):
+        def _(self, *args):
+            retval = parser.formatter._format_option_strings(*args)
+            if len(retval) > MAX_HELP_OPTION_LENGTH:
+                retval = ("%%.%ds.." % (MAX_HELP_OPTION_LENGTH - parser.formatter.indent_increment)) % retval
+            return retval.capitalize()
+
+        parser.formatter._format_option_strings = parser.formatter.format_option_strings
+        parser.formatter.format_option_strings = type(parser.formatter.format_option_strings)(_, parser)
+    else:
+        def _format_action_invocation(self, action):
+            retval = self.__format_action_invocation(action)
+            if len(retval) > MAX_HELP_OPTION_LENGTH:
+                retval = ("%%.%ds.." % (MAX_HELP_OPTION_LENGTH - self._indent_increment)) % retval
+            return retval.capitalize()
+
+        parser.formatter_class.__format_action_invocation = parser.formatter_class._format_action_invocation
+        parser.formatter_class._format_action_invocation = _format_action_invocation
