@@ -12,9 +12,65 @@
 
 namespace PHP_CodeSniffer;
 
-use PHP_CodeSniffer\Exceptions\RuntimeException;
 use PHP_CodeSniffer\Exceptions\DeepExitException;
+use PHP_CodeSniffer\Exceptions\RuntimeException;
 
+/**
+ * Stores the configuration used to run PHPCS and PHPCBF.
+ *
+ * @property string[] $files           The files and directories to check.
+ * @property string[] $standards       The standards being used for checking.
+ * @property int      $verbosity       How verbose the output should be.
+ *                                     0: no unnecessary output
+ *                                     1: basic output for files being checked
+ *                                     2: ruleset and file parsing output
+ *                                     3: sniff execution output
+ * @property bool     $interactive     Enable interactive checking mode.
+ * @property bool     $parallel        Check files in parallel.
+ * @property bool     $cache           Enable the use of the file cache.
+ * @property bool     $cacheFile       A file where the cache data should be written
+ * @property bool     $colors          Display colours in output.
+ * @property bool     $explain         Explain the coding standards.
+ * @property bool     $local           Process local files in directories only (no recursion).
+ * @property bool     $showSources     Show sniff source codes in report output.
+ * @property bool     $showProgress    Show basic progress information while running.
+ * @property bool     $quiet           Quiet mode; disables progress and verbose output.
+ * @property bool     $annotations     Process phpcs: annotations.
+ * @property int      $tabWidth        How many spaces each tab is worth.
+ * @property string   $encoding        The encoding of the files being checked.
+ * @property string[] $sniffs          The sniffs that should be used for checking.
+ *                                     If empty, all sniffs in the supplied standards will be used.
+ * @property string[] $exclude         The sniffs that should be excluded from checking.
+ *                                     If empty, all sniffs in the supplied standards will be used.
+ * @property string[] $ignored         Regular expressions used to ignore files and folders during checking.
+ * @property string   $reportFile      A file where the report output should be written.
+ * @property string   $generator       The documentation generator to use.
+ * @property string   $filter          The filter to use for the run.
+ * @property string[] $bootstrap       One of more files to include before the run begins.
+ * @property int      $reportWidth     The maximum number of columns that reports should use for output.
+ *                                     Set to "auto" for have this value changed to the width of the terminal.
+ * @property int      $errorSeverity   The minimum severity an error must have to be displayed.
+ * @property int      $warningSeverity The minimum severity a warning must have to be displayed.
+ * @property bool     $recordErrors    Record the content of error messages as well as error counts.
+ * @property string   $suffix          A suffix to add to fixed files.
+ * @property string   $basepath        A file system location to strip from the paths of files shown in reports.
+ * @property bool     $stdin           Read content from STDIN instead of supplied files.
+ * @property string   $stdinContent    Content passed directly to PHPCS on STDIN.
+ * @property string   $stdinPath       The path to use for content passed on STDIN.
+ *
+ * @property array<string, string>      $extensions File extensions that should be checked, and what tokenizer to use.
+ *                                                  E.g., array('inc' => 'PHP');
+ * @property array<string, string|null> $reports    The reports to use for printing output after the run.
+ *                                                  The format of the array is:
+ *                                                      array(
+ *                                                          'reportName1' => 'outputFile',
+ *                                                          'reportName2' => null,
+ *                                                      );
+ *                                                  If the array value is NULL, the report will be written to the screen.
+ *
+ * @property string[] $unknown Any arguments gathered on the command line that are unknown to us.
+ *                             E.g., using `phpcs -c` will give array('c');
+ */
 class Config
 {
 
@@ -23,7 +79,7 @@ class Config
      *
      * @var string
      */
-    const VERSION = '3.3.0';
+    const VERSION = '3.6.0';
 
     /**
      * Package stability; either stable, beta or alpha.
@@ -40,60 +96,7 @@ class Config
      * can be used to validate the values. For example, to set the verbosity level to
      * level 2, use $this->verbosity = 2; instead of accessing this property directly.
      *
-     * The list of settings are:
-     *
-     * string[] files           The files and directories to check.
-     * string[] standards       The standards being used for checking.
-     * int      verbosity       How verbose the output should be.
-     *                          0: no unnecessary output
-     *                          1: basic output for files being checked
-     *                          2: ruleset and file parsing output
-     *                          3: sniff execution output
-     * bool     interactive     Enable interactive checking mode.
-     * bool     parallel        Check files in parallel.
-     * bool     cache           Enable the use of the file cache.
-     * bool     cacheFile       A file where the cache data should be written
-     * bool     colors          Display colours in output.
-     * bool     explain         Explain the coding standards.
-     * bool     local           Process local files in directories only (no recursion).
-     * bool     showSources     Show sniff source codes in report output.
-     * bool     showProgress    Show basic progress information while running.
-     * bool     quiet           Quiet mode; disables progress and verbose output.
-     * bool     annotations     Process phpcs: annotations.
-     * int      tabWidth        How many spaces each tab is worth.
-     * string   encoding        The encoding of the files being checked.
-     * string[] sniffs          The sniffs that should be used for checking.
-     *                          If empty, all sniffs in the supplied standards will be used.
-     * string[] exclude         The sniffs that should be excluded from checking.
-     *                          If empty, all sniffs in the supplied standards will be used.
-     * string[] ignored         Regular expressions used to ignore files and folders during checking.
-     * string   reportFile      A file where the report output should be written.
-     * string   generator       The documentation generator to use.
-     * string   filter          The filter to use for the run.
-     * string[] bootstrap       One of more files to include before the run begins.
-     * int      reportWidth     The maximum number of columns that reports should use for output.
-     *                          Set to "auto" for have this value changed to the width of the terminal.
-     * int      errorSeverity   The minimum severity an error must have to be displayed.
-     * int      warningSeverity The minimum severity a warning must have to be displayed.
-     * bool     recordErrors    Record the content of error messages as well as error counts.
-     * string   suffix          A suffix to add to fixed files.
-     * string   basepath        A file system location to strip from the paths of files shown in reports.
-     * bool     stdin           Read content from STDIN instead of supplied files.
-     * string   stdinContent    Content passed directly to PHPCS on STDIN.
-     * string   stdinPath       The path to use for content passed on STDIN.
-     *
-     * array<string, string>      extensions File extensions that should be checked, and what tokenizer to use.
-     *                                       E.g., array('inc' => 'PHP');
-     * array<string, string|null> reports    The reports to use for printing output after the run.
-     *                                       The format of the array is:
-     *                                           array(
-     *                                            'reportName1' => 'outputFile',
-     *                                            'reportName2' => null,
-     *                                           );
-     *                                       If the array value is NULL, the report will be written to the screen.
-     *
-     * string[] unknown Any arguments gathered on the command line that are unknown to us.
-     *                  E.g., using `phpcs -c` will give array('c');
+     * Each of these settings is described in the class comment property list.
      *
      * @var array<string, mixed>
      */
@@ -188,7 +191,7 @@ class Config
      * @param string $name The name of the property.
      *
      * @return mixed
-     * @throws RuntimeException If the setting name is invalid.
+     * @throws \PHP_CodeSniffer\Exceptions\RuntimeException If the setting name is invalid.
      */
     public function __get($name)
     {
@@ -208,7 +211,7 @@ class Config
      * @param mixed  $value The value of the property.
      *
      * @return void
-     * @throws RuntimeException If the setting name is invalid.
+     * @throws \PHP_CodeSniffer\Exceptions\RuntimeException If the setting name is invalid.
      */
     public function __set($name, $value)
     {
@@ -219,7 +222,10 @@ class Config
         switch ($name) {
         case 'reportWidth' :
             // Support auto terminal width.
-            if ($value === 'auto' && preg_match('|\d+ (\d+)|', shell_exec('stty size 2>&1'), $matches) === 1) {
+            if ($value === 'auto'
+                && function_exists('shell_exec') === true
+                && preg_match('|\d+ (\d+)|', shell_exec('stty size 2>&1'), $matches) === 1
+            ) {
                 $value = (int) $matches[1];
             } else {
                 $value = (int) $value;
@@ -357,7 +363,7 @@ class Config
 
                 $lastDir    = $currentDir;
                 $currentDir = dirname($currentDir);
-            } while ($currentDir !== '.' && $currentDir !== $lastDir);
+            } while ($currentDir !== '.' && $currentDir !== $lastDir && @is_readable($currentDir) === true);
         }//end if
 
         if (defined('STDIN') === false
@@ -394,12 +400,6 @@ class Config
             }
         }//end if
 
-        if (defined('PHP_CODESNIFFER_IN_TESTS') === true && defined('HHVM_VERSION') === true) {
-            // HHVM 3.25+ wont let us re-open STDIN after it is closed.
-            // So don't close it.
-            return;
-        }
-
         fclose($handle);
 
     }//end __construct()
@@ -423,7 +423,7 @@ class Config
                 continue;
             }
 
-            if ($arg{0} === '-') {
+            if ($arg[0] === '-') {
                 if ($arg === '-') {
                     // Asking to read from STDIN.
                     $this->stdin = true;
@@ -436,7 +436,7 @@ class Config
                     continue;
                 }
 
-                if ($arg{1} === '-') {
+                if ($arg[1] === '-') {
                     $this->processLongArgument(substr($arg, 2), $i);
                 } else {
                     $switches = str_split($arg);
@@ -590,6 +590,7 @@ class Config
      * @param int    $pos The position of the argument on the command line.
      *
      * @return void
+     * @throws \PHP_CodeSniffer\Exceptions\DeepExitException
      */
     public function processShortArgument($arg, $pos)
     {
@@ -694,6 +695,7 @@ class Config
      * @param int    $pos The position of the argument on the command line.
      *
      * @return void
+     * @throws \PHP_CodeSniffer\Exceptions\DeepExitException
      */
     public function processLongArgument($arg, $pos)
     {
@@ -892,7 +894,7 @@ class Config
                         // Passed cache file is a file in the current directory.
                         $this->cacheFile = getcwd().'/'.basename($this->cacheFile);
                     } else {
-                        if ($dir{0} === '/') {
+                        if ($dir[0] === '/') {
                             // An absolute path.
                             $dir = Util\Common::realpath($dir);
                         } else {
@@ -973,29 +975,14 @@ class Config
                 if ($this->reportFile === false) {
                     $this->reportFile = substr($arg, 12);
 
-                    $dir = dirname($this->reportFile);
+                    $dir = Util\Common::realpath(dirname($this->reportFile));
                     if (is_dir($dir) === false) {
                         $error  = 'ERROR: The specified report file path "'.$this->reportFile.'" points to a non-existent directory'.PHP_EOL.PHP_EOL;
                         $error .= $this->printShortUsage(true);
                         throw new DeepExitException($error, 3);
                     }
 
-                    if ($dir === '.') {
-                        // Passed report file is a file in the current directory.
-                        $this->reportFile = getcwd().'/'.basename($this->reportFile);
-                    } else {
-                        if ($dir{0} === '/') {
-                            // An absolute path.
-                            $dir = Util\Common::realpath($dir);
-                        } else {
-                            $dir = Util\Common::realpath(getcwd().'/'.$dir);
-                        }
-
-                        if ($dir !== false) {
-                            // Report file path is relative.
-                            $this->reportFile = $dir.'/'.basename($this->reportFile);
-                        }
-                    }
+                    $this->reportFile = $dir.'/'.basename($this->reportFile);
                 }//end if
 
                 self::$overriddenDefaults['reportFile'] = true;
@@ -1051,28 +1038,19 @@ class Config
                         if ($output === false) {
                             $output = null;
                         } else {
-                            $dir = dirname($output);
+                            $dir = Util\Common::realpath(dirname($output));
                             if (is_dir($dir) === false) {
                                 $error  = 'ERROR: The specified '.$report.' report file path "'.$output.'" points to a non-existent directory'.PHP_EOL.PHP_EOL;
                                 $error .= $this->printShortUsage(true);
                                 throw new DeepExitException($error, 3);
                             }
 
-                            if ($dir === '.') {
-                                // Passed report file is a filename in the current directory.
-                                $output = getcwd().'/'.basename($output);
-                            } else {
-                                if ($dir{0} === '/') {
-                                    // An absolute path.
-                                    $dir = Util\Common::realpath($dir);
-                                } else {
-                                    $dir = Util\Common::realpath(getcwd().'/'.$dir);
-                                }
+                            $output = $dir.'/'.basename($output);
 
-                                if ($dir !== false) {
-                                    // Report file path is relative.
-                                    $output = $dir.'/'.basename($output);
-                                }
+                            if (is_dir($output) === true) {
+                                $error  = 'ERROR: The specified '.$report.' report file path "'.$output.'" is a directory'.PHP_EOL.PHP_EOL;
+                                $error .= $this->printShortUsage(true);
+                                throw new DeepExitException($error, 3);
                             }
                         }//end if
                     }//end if
@@ -1255,11 +1233,12 @@ class Config
      * @param int    $pos The position of the argument on the command line.
      *
      * @return void
+     * @throws \PHP_CodeSniffer\Exceptions\DeepExitException
      */
     public function processUnknownArgument($arg, $pos)
     {
         // We don't know about any additional switches; just files.
-        if ($arg{0} === '-') {
+        if ($arg[0] === '-') {
             if ($this->dieOnUnknownArg === false) {
                 return;
             }
@@ -1280,6 +1259,7 @@ class Config
      * @param string $path The path to the file to add.
      *
      * @return void
+     * @throws \PHP_CodeSniffer\Exceptions\DeepExitException
      */
     public function processFilePath($path)
     {
@@ -1298,6 +1278,8 @@ class Config
             $error .= $this->printShortUsage(true);
             throw new DeepExitException($error, 3);
         } else {
+            // Can't modify the files array directly because it's not a real
+            // class member, so need to use this little get/modify/set trick.
             $files       = $this->files;
             $files[]     = $file;
             $this->files = $files;
@@ -1370,7 +1352,7 @@ class Config
         echo '  [--standard=<standard>] [--sniffs=<sniffs>] [--exclude=<sniffs>]'.PHP_EOL;
         echo '  [--encoding=<encoding>] [--parallel=<processes>] [--generator=<generator>]'.PHP_EOL;
         echo '  [--extensions=<extensions>] [--ignore=<patterns>] [--ignore-annotations]'.PHP_EOL;
-        echo '  [--stdin-path=<stdinPath>] [--file-list=<fileList>] <file> - ...'.PHP_EOL;
+        echo '  [--stdin-path=<stdinPath>] [--file-list=<fileList>] [--filter=<filter>] <file> - ...'.PHP_EOL;
         echo PHP_EOL;
         echo ' -     Check STDIN instead of local files and directories'.PHP_EOL;
         echo ' -n    Do not print warnings (shortcut for --warning-severity=0)'.PHP_EOL;
@@ -1400,19 +1382,22 @@ class Config
         echo ' <cacheFile>    Use a specific file for caching (uses a temporary file by default)'.PHP_EOL;
         echo ' <basepath>     A path to strip from the front of file paths inside reports'.PHP_EOL;
         echo ' <bootstrap>    A comma separated list of files to run before processing begins'.PHP_EOL;
-        echo ' <file>         One or more files and/or directories to check'.PHP_EOL;
-        echo ' <fileList>     A file containing a list of files and/or directories to check (one per line)'.PHP_EOL;
         echo ' <encoding>     The encoding of the files being checked (default is utf-8)'.PHP_EOL;
         echo ' <extensions>   A comma separated list of file extensions to check'.PHP_EOL;
         echo '                The type of the file can be specified using: ext/type'.PHP_EOL;
         echo '                e.g., module/php,es/js'.PHP_EOL;
-        echo ' <generator>    Uses either the "HTML", "Markdown" or "Text" generator'.PHP_EOL;
+        echo ' <file>         One or more files and/or directories to check'.PHP_EOL;
+        echo ' <fileList>     A file containing a list of files and/or directories to check (one per line)'.PHP_EOL;
+        echo ' <filter>       Use either the "gitmodified" or "gitstaged" filter,'.PHP_EOL;
+        echo '                or specify the path to a custom filter class'.PHP_EOL;
+        echo ' <generator>    Use either the "HTML", "Markdown" or "Text" generator'.PHP_EOL;
         echo '                (forces documentation generation instead of checking)'.PHP_EOL;
         echo ' <patterns>     A comma separated list of patterns to ignore files and directories'.PHP_EOL;
         echo ' <processes>    How many files should be checked simultaneously (default is 1)'.PHP_EOL;
         echo ' <report>       Print either the "full", "xml", "checkstyle", "csv"'.PHP_EOL;
         echo '                "json", "junit", "emacs", "source", "summary", "diff"'.PHP_EOL;
-        echo '                "svnblame", "gitblame", "hgblame" or "notifysend" report'.PHP_EOL;
+        echo '                "svnblame", "gitblame", "hgblame" or "notifysend" report,'.PHP_EOL;
+        echo '                or specify the path to a custom report class'.PHP_EOL;
         echo '                (the "full" report is printed by default)'.PHP_EOL;
         echo ' <reportFile>   Write the report to the specified file path'.PHP_EOL;
         echo ' <reportWidth>  How many columns wide screen reports should be printed'.PHP_EOL;
@@ -1439,7 +1424,7 @@ class Config
         echo '  [--severity=<severity>] [--error-severity=<severity>] [--warning-severity=<severity>]'.PHP_EOL;
         echo '  [--tab-width=<tabWidth>] [--encoding=<encoding>] [--parallel=<processes>]'.PHP_EOL;
         echo '  [--basepath=<basepath>] [--extensions=<extensions>] [--ignore=<patterns>]'.PHP_EOL;
-        echo '  [--stdin-path=<stdinPath>] [--file-list=<fileList>] <file> - ...'.PHP_EOL;
+        echo '  [--stdin-path=<stdinPath>] [--file-list=<fileList>] [--filter=<filter>] <file> - ...'.PHP_EOL;
         echo PHP_EOL;
         echo ' -     Fix STDIN instead of local files and directories'.PHP_EOL;
         echo ' -n    Do not fix warnings (shortcut for --warning-severity=0)'.PHP_EOL;
@@ -1459,12 +1444,14 @@ class Config
         echo PHP_EOL;
         echo ' <basepath>    A path to strip from the front of file paths inside reports'.PHP_EOL;
         echo ' <bootstrap>   A comma separated list of files to run before processing begins'.PHP_EOL;
-        echo ' <file>        One or more files and/or directories to fix'.PHP_EOL;
-        echo ' <fileList>    A file containing a list of files and/or directories to fix (one per line)'.PHP_EOL;
         echo ' <encoding>    The encoding of the files being fixed (default is utf-8)'.PHP_EOL;
         echo ' <extensions>  A comma separated list of file extensions to fix'.PHP_EOL;
         echo '               The type of the file can be specified using: ext/type'.PHP_EOL;
         echo '               e.g., module/php,es/js'.PHP_EOL;
+        echo ' <file>        One or more files and/or directories to fix'.PHP_EOL;
+        echo ' <fileList>    A file containing a list of files and/or directories to fix (one per line)'.PHP_EOL;
+        echo ' <filter>      Use either the "gitmodified" or "gitstaged" filter,'.PHP_EOL;
+        echo '               or specify the path to a custom filter class'.PHP_EOL;
         echo ' <patterns>    A comma separated list of patterns to ignore files and directories'.PHP_EOL;
         echo ' <processes>   How many files should be fixed simultaneously (default is 1)'.PHP_EOL;
         echo ' <severity>    The minimum severity required to fix an error or warning'.PHP_EOL;
@@ -1520,6 +1507,11 @@ class Config
             return $data;
         }
 
+        if ($name === "php") {
+            // For php, we know the executable path. There's no need to look it up.
+            return PHP_BINARY;
+        }
+
         if (array_key_exists($name, self::$executablePaths) === true) {
             return self::$executablePaths[$name];
         }
@@ -1554,7 +1546,7 @@ class Config
      *
      * @return bool
      * @see    getConfigData()
-     * @throws RuntimeException If the config file can not be written.
+     * @throws \PHP_CodeSniffer\Exceptions\DeepExitException If the config file can not be written.
      */
     public static function setConfigData($key, $value, $temp=false)
     {
@@ -1635,6 +1627,7 @@ class Config
      *
      * @return array<string, string>
      * @see    getConfigData()
+     * @throws \PHP_CodeSniffer\Exceptions\DeepExitException If the config file could not be read.
      */
     public static function getAllConfigData()
     {
