@@ -34,6 +34,7 @@ define('T_MODULUS', 'PHPCS_T_MODULUS');
 define('T_BITWISE_AND', 'PHPCS_T_BITWISE_AND');
 define('T_BITWISE_OR', 'PHPCS_T_BITWISE_OR');
 define('T_BITWISE_XOR', 'PHPCS_T_BITWISE_XOR');
+define('T_BITWISE_NOT', 'PHPCS_T_BITWISE_NOT');
 define('T_ARRAY_HINT', 'PHPCS_T_ARRAY_HINT');
 define('T_GREATER_THAN', 'PHPCS_T_GREATER_THAN');
 define('T_LESS_THAN', 'PHPCS_T_LESS_THAN');
@@ -73,6 +74,12 @@ define('T_OPEN_USE_GROUP', 'PHPCS_T_OPEN_USE_GROUP');
 define('T_CLOSE_USE_GROUP', 'PHPCS_T_CLOSE_USE_GROUP');
 define('T_ZSR', 'PHPCS_T_ZSR');
 define('T_ZSR_EQUAL', 'PHPCS_T_ZSR_EQUAL');
+define('T_FN_ARROW', 'PHPCS_T_FN_ARROW');
+define('T_TYPE_UNION', 'PHPCS_T_TYPE_UNION');
+define('T_PARAM_NAME', 'PHPCS_T_PARAM_NAME');
+define('T_MATCH_ARROW', 'PHPCS_T_MATCH_ARROW');
+define('T_MATCH_DEFAULT', 'PHPCS_T_MATCH_DEFAULT');
+define('T_ATTRIBUTE_END', 'PHPCS_T_ATTRIBUTE_END');
 
 // Some PHP 5.5 tokens, replicated for lower versions.
 if (defined('T_FINALLY') === false) {
@@ -113,6 +120,40 @@ if (defined('T_YIELD_FROM') === false) {
     define('T_YIELD_FROM', 'PHPCS_T_YIELD_FROM');
 }
 
+// Some PHP 7.4 tokens, replicated for lower versions.
+if (defined('T_BAD_CHARACTER') === false) {
+    define('T_BAD_CHARACTER', 'PHPCS_T_BAD_CHARACTER');
+}
+
+if (defined('T_FN') === false) {
+    define('T_FN', 'PHPCS_T_FN');
+}
+
+// Some PHP 8.0 tokens, replicated for lower versions.
+if (defined('T_NULLSAFE_OBJECT_OPERATOR') === false) {
+    define('T_NULLSAFE_OBJECT_OPERATOR', 'PHPCS_T_NULLSAFE_OBJECT_OPERATOR');
+}
+
+if (defined('T_NAME_QUALIFIED') === false) {
+    define('T_NAME_QUALIFIED', 'PHPCS_T_NAME_QUALIFIED');
+}
+
+if (defined('T_NAME_FULLY_QUALIFIED') === false) {
+    define('T_NAME_FULLY_QUALIFIED', 'PHPCS_T_NAME_FULLY_QUALIFIED');
+}
+
+if (defined('T_NAME_RELATIVE') === false) {
+    define('T_NAME_RELATIVE', 'PHPCS_T_NAME_RELATIVE');
+}
+
+if (defined('T_MATCH') === false) {
+    define('T_MATCH', 'PHPCS_T_MATCH');
+}
+
+if (defined('T_ATTRIBUTE') === false) {
+    define('T_ATTRIBUTE', 'PHPCS_T_ATTRIBUTE');
+}
+
 // Tokens used for parsing doc blocks.
 define('T_DOC_COMMENT_STAR', 'PHPCS_T_DOC_COMMENT_STAR');
 define('T_DOC_COMMENT_WHITESPACE', 'PHPCS_T_DOC_COMMENT_WHITESPACE');
@@ -134,7 +175,7 @@ final class Tokens
     /**
      * The token weightings.
      *
-     * @var array<int, int>
+     * @var array<int|string, int>
      */
     public static $weightings = [
         T_CLASS               => 1000,
@@ -144,9 +185,9 @@ final class Tokens
         T_FUNCTION            => 100,
         T_CLOSURE             => 100,
 
-                                 /*
-                                     Conditions.
-                                 */
+        /*
+         * Conditions.
+         */
 
         T_WHILE               => 50,
         T_FOR                 => 50,
@@ -159,13 +200,14 @@ final class Tokens
         T_CATCH               => 50,
         T_FINALLY             => 50,
         T_SWITCH              => 50,
+        T_MATCH               => 50,
 
         T_SELF                => 25,
         T_PARENT              => 25,
 
-                                 /*
-                                     Operators and arithmetic.
-                                 */
+        /*
+         * Operators and arithmetic.
+         */
 
         T_BITWISE_AND         => 8,
         T_BITWISE_OR          => 8,
@@ -200,9 +242,9 @@ final class Tokens
         T_BOOLEAN_AND         => 5,
         T_BOOLEAN_OR          => 5,
 
-                                 /*
-                                     Equality.
-                                 */
+        /*
+         * Equality.
+         */
 
         T_IS_EQUAL            => 5,
         T_IS_NOT_EQUAL        => 5,
@@ -215,7 +257,7 @@ final class Tokens
     /**
      * Tokens that represent assignments.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $assignmentTokens = [
         T_EQUAL          => T_EQUAL,
@@ -239,7 +281,7 @@ final class Tokens
     /**
      * Tokens that represent equality comparisons.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $equalityTokens = [
         T_IS_EQUAL            => T_IS_EQUAL,
@@ -253,7 +295,7 @@ final class Tokens
     /**
      * Tokens that represent comparison operator.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $comparisonTokens = [
         T_IS_EQUAL            => T_IS_EQUAL,
@@ -271,7 +313,7 @@ final class Tokens
     /**
      * Tokens that represent arithmetic operators.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $arithmeticTokens = [
         T_PLUS     => T_PLUS,
@@ -285,7 +327,7 @@ final class Tokens
     /**
      * Tokens that perform operations.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $operators = [
         T_MINUS       => T_MINUS,
@@ -306,7 +348,7 @@ final class Tokens
     /**
      * Tokens that perform boolean operations.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $booleanOperators = [
         T_BOOLEAN_AND => T_BOOLEAN_AND,
@@ -319,7 +361,7 @@ final class Tokens
     /**
      * Tokens that represent casting.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $castTokens = [
         T_INT_CAST    => T_INT_CAST,
@@ -335,26 +377,29 @@ final class Tokens
     /**
      * Token types that open parenthesis.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $parenthesisOpeners = [
-        T_ARRAY    => T_ARRAY,
-        T_FUNCTION => T_FUNCTION,
-        T_CLOSURE  => T_CLOSURE,
-        T_WHILE    => T_WHILE,
-        T_FOR      => T_FOR,
-        T_FOREACH  => T_FOREACH,
-        T_SWITCH   => T_SWITCH,
-        T_IF       => T_IF,
-        T_ELSEIF   => T_ELSEIF,
-        T_CATCH    => T_CATCH,
-        T_DECLARE  => T_DECLARE,
+        T_ARRAY      => T_ARRAY,
+        T_LIST       => T_LIST,
+        T_FUNCTION   => T_FUNCTION,
+        T_CLOSURE    => T_CLOSURE,
+        T_ANON_CLASS => T_ANON_CLASS,
+        T_WHILE      => T_WHILE,
+        T_FOR        => T_FOR,
+        T_FOREACH    => T_FOREACH,
+        T_SWITCH     => T_SWITCH,
+        T_IF         => T_IF,
+        T_ELSEIF     => T_ELSEIF,
+        T_CATCH      => T_CATCH,
+        T_DECLARE    => T_DECLARE,
+        T_MATCH      => T_MATCH,
     ];
 
     /**
      * Tokens that are allowed to open scopes.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $scopeOpeners = [
         T_CLASS      => T_CLASS,
@@ -381,12 +426,13 @@ final class Tokens
         T_PROPERTY   => T_PROPERTY,
         T_OBJECT     => T_OBJECT,
         T_USE        => T_USE,
+        T_MATCH      => T_MATCH,
     ];
 
     /**
      * Tokens that represent scope modifiers.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $scopeModifiers = [
         T_PRIVATE   => T_PRIVATE,
@@ -397,7 +443,7 @@ final class Tokens
     /**
      * Tokens that can prefix a method name
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $methodPrefixes = [
         T_PRIVATE   => T_PRIVATE,
@@ -411,7 +457,7 @@ final class Tokens
     /**
      * Tokens that open code blocks.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $blockOpeners = [
         T_OPEN_CURLY_BRACKET  => T_OPEN_CURLY_BRACKET,
@@ -423,7 +469,7 @@ final class Tokens
     /**
      * Tokens that don't represent code.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $emptyTokens = [
         T_WHITESPACE             => T_WHITESPACE,
@@ -445,7 +491,7 @@ final class Tokens
     /**
      * Tokens that are comments.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $commentTokens = [
         T_COMMENT                => T_COMMENT,
@@ -466,7 +512,7 @@ final class Tokens
     /**
      * Tokens that are comments containing PHPCS instructions.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $phpcsCommentTokens = [
         T_PHPCS_ENABLE      => T_PHPCS_ENABLE,
@@ -481,7 +527,7 @@ final class Tokens
      *
      * Note that T_STRINGS are NOT represented in this list.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $stringTokens = [
         T_CONSTANT_ENCAPSED_STRING => T_CONSTANT_ENCAPSED_STRING,
@@ -491,7 +537,7 @@ final class Tokens
     /**
      * Tokens that represent text strings.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $textStringTokens = [
         T_CONSTANT_ENCAPSED_STRING => T_CONSTANT_ENCAPSED_STRING,
@@ -504,7 +550,7 @@ final class Tokens
     /**
      * Tokens that represent brackets and parenthesis.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $bracketTokens = [
         T_OPEN_CURLY_BRACKET   => T_OPEN_CURLY_BRACKET,
@@ -518,7 +564,7 @@ final class Tokens
     /**
      * Tokens that include files.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $includeTokens = [
         T_REQUIRE_ONCE => T_REQUIRE_ONCE,
@@ -530,7 +576,7 @@ final class Tokens
     /**
      * Tokens that make up a heredoc string.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $heredocTokens = [
         T_START_HEREDOC => T_START_HEREDOC,
@@ -547,7 +593,7 @@ final class Tokens
      * Mostly, these are just strings. But PHP tokenizes some language
      * constructs and functions using their own tokens.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $functionNameTokens = [
         T_STRING       => T_STRING,
@@ -565,9 +611,9 @@ final class Tokens
     ];
 
     /**
-     * Tokens that are open class and object scopes.
+     * Tokens that open class and object scopes.
      *
-     * @var array<int, int>
+     * @var array<int|string, int|string>
      */
     public static $ooScopeTokens = [
         T_CLASS      => T_CLASS,
@@ -576,6 +622,24 @@ final class Tokens
         T_TRAIT      => T_TRAIT,
     ];
 
+    /**
+     * Tokens representing PHP magic constants.
+     *
+     * @var array <int|string> => <int|string>
+     *
+     * @link https://www.php.net/language.constants.predefined PHP Manual on magic constants
+     */
+    public static $magicConstants = [
+        T_CLASS_C  => T_CLASS_C,
+        T_DIR      => T_DIR,
+        T_FILE     => T_FILE,
+        T_FUNC_C   => T_FUNC_C,
+        T_LINE     => T_LINE,
+        T_METHOD_C => T_METHOD_C,
+        T_NS_C     => T_NS_C,
+        T_TRAIT_C  => T_TRAIT_C,
+    ];
+
 
     /**
      * Given a token, returns the name of the token.
@@ -610,10 +674,10 @@ final class Tokens
      *
      * Returns false if there are no weightings for any of the specified tokens.
      *
-     * @param array<int, int> $tokens The token types to get the highest weighted
-     *                                type for.
+     * @param array<int|string> $tokens The token types to get the highest weighted
+     *                                  type for.
      *
-     * @return int The highest weighted token.
+     * @return int|false The highest weighted token.
      */
     public static function getHighestWeightedToken(array $tokens)
     {
